# This directory order is carefully chosen:
# * We use programs compiled in native/constgen to generate sources
#   in src, src/syslib/user/ovm_platform, and native/common
# * src/syslib/user depends on src/syslib/user/ovm_platform
# * We cannot compile native/common until header files have been
#   generated by s3.services.buildtools.DumpInterface (which is part
#   of the main source directory)..
# * We cannot compile src/apps until the jars in src/syslib/user are available
# * src/syslib/s3 depends on src, rather than the other way around
# * bin can be mad at any time, but we don't don't need it until
#   we want to run `make check' or `make regression-test'
# * src/native/{interpreter,simplejit,j2c} and build have nothing for all, 
#   they install their sources for use with gen-ovm.
# * libffi is not includeded in the list because install must be
#   handled specially
SUBDIRS=src/native/constgen			\
	src/syslib/user/ovm_platform		\
	src/syslib/user				\
        src/syslib/user/ovm_realtime            \
	src					\
	src/syslib/s3				\
	src/native/common/include/jvm_constants	\
	src/native/common			\
	src/apps				\
	bin					\
	src/native/interpreter			\
	src/native/simplejit			\
	src/native/j2c				\
	config					\
	build					\
	doc/manual

all: all-recursive 
	$(MAKE) -w -C libffi/libffi $@

# Update ovmj.org with generated files (webinstall should be run as ovm).
weball webinstall:
	$(MAKE) -w -C doc/manual $@
	$(MAKE) -w -C src $@

# Update CVS copies of generated files
distall distinstall:
	$(MAKE) -w -C doc/manual $@

# give libffi's clean some help
clean: clean-recursive
	-$(MAKE) -w -C libffi/libffi $@
	-rm -f libffi/libffi/src/*.o
	-rm -f libffi/libffi/src/*.lo
	-rm -f libffi/libffi/src/x86/*.o
	-rm -f libffi/libffi/src/x86/*.lo
	-rm -f libffi/libffi/src/arm/*.o
	-rm -f libffi/libffi/src/arm/*.lo
	-rm -f libffi/libffi/src/powerpc/*.o
	-rm -f libffi/libffi/src/powerpc/*.lo
	$(MAKE) -w -C test $@

check: all
	$(MAKE) -w -C test $@

regression-test: all
	$(MAKE) -w -C test $@

# give libffi's distclean some help
distclean: clean
	-$(MAKE) -w -C libffi/libffi distclean-am distclean
	-$(MAKE) -w -C libffi/libffi/include distclean
	-rm -f libffi/libffi/config.status	 \
	       libffi/libffi/include/fficonfig.h
	-rm  bin/gen-ovm				      \
	     bin/Makefile				      \
	     config/paths				      \
	     config/Makefile				      \
	     build/Vars.mk				      \
	     build/Makefile				      \
	     Makefile					      \
	     doc/manual/Makefile			      \
	     src/Makefile				      \
	     src/apps/Makefile				      \
	     src/native/constgen/Makefile		      \
	     src/native/common/Makefile			      \
	     src/native/common/include/autodefs.h	      \
	     src/native/common/include/jvm_constants/Makefile \
	     src/native/interpreter/Makefile		      \
	     src/native/simplejit/Makefile		      \
	     src/native/j2c/Makefile			      \
	     src/syslib/s3/Makefile			      \
	     src/syslib/user/Makefile			      \
	     src/syslib/user/ovm_platform/Makefile	      \
	     src/syslib/user/ovm_realtime/Makefile	      \
	     test/Makefile				      \
	     config.status				      \
	     config.log

install: install-recursive
	$(MAKE) -w -C libffi/libffi prefix=$(prefix)/ovm install


aspectcheck:
	$(MAKE) -w -C src aspectcheck
