# Rules for compiling a j2c-based virual machine
INCS +=  -I. -I$(srcdir) $(include_path)
# I had src/.libs here initially, but it didn't work!
LDFLAGS += $(lib_path)
LIBS:=-lovmnative -lffi $(LIBS)

DRIVER_GENERATED+=ovm_inline.c 		\
		  ovm_inline.h			\
		  ovm_heap_exports.s		\
		  ovm_heap_exports.h

# Turn off warnings in this directory, and be VERY sure to compile C code
# with exception support
CFLAGS:=$(patsubst -W%,,$(CFLAGS)) -fno-strict-aliasing

CXX_SRC=jrt.c
C_SRC=invoke_native.c gen-ovm.c
ASM_SRC=ovm_heap_exports.s

include GenOvm.mk

OBJS=$(CXX_SRC:.c=.o) $(C_SRC:.c=.o) $(ASM_SRC:.s=.o) \
     ovm_inline_tables.o $(IMAGE_LINK_MAGIC_FILE)

ovm_inline.s: ovm_inline.i
	$(CC) $(ALL_CXXFLAGS) -fpreprocessed -S -o $@ $<

# This target implicitly generates the file code_indexes.
# code_indexes contains one line per J2cCodeFragment.frags entry,
# giving the method UID for this code-fragment index.
ovm_inline_tables.s: ovm_inline.s make-method-table.awk
	$(FILTER_OVM_INLINE) |\
	$(AWK) -v header_skip_bytes=$(HEADER_SKIP_BYTES) \
	       -v underscore=$(UNDERSCORE) \
	       -f $(srcdir)/make-method-table.awk  > $@

# Under Darwin, we need an image file in order to link ovm_raw.  No
# image file means no bootimage section.  No bootimage section means a
# smaller mach-O header.  A smaller mach-O header changes addresses in
# the rest of the executable, and renders an image generated against
# such an ovm_raw useless.
img:
	echo "dummy bootimage contents" > $@

# print exported data addresss in the format XXXXXXXX S*
ovm_heap_imports: ovm
	$(NM) $< | $(AWK) '/^[0-9a-fA-F]/ && $$2 ~ /^D$$/ {		\
	    sub("^00000000", "");					\
            printf("%s %s\n", $$1, $$3);				\
         }' > $@

ovm: $(OBJS) img
	$(CC) $(APP_LDFLAGS) $(LDFLAGS) $(OPT) -o $@ \
	       $(OBJS) $(LIBS) $(APP_LIBS)

clean: native-clean
	-rm $(DRIVER_GENERATED) ovm ovm_heap_imports

