
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <assert.h>
#include <sys/mman.h>
#include <errno.h>
#include <stdarg.h>
#include <sys/io.h>

// Xenomai
#include <string.h>
//#include <signal.h>
#include <execinfo.h>

#ifdef OVM_XENOMAI
#include <native/task.h>
#include <rtdk.h>
#endif

#include "config.h"

// on Linux, ucontext.h must be included with __USE_GNU (which must be
// turned on by _GNU_SOURCE) to support register access constants like
// REG_EIP if ucontext is included recursively before, we are screwed

#ifdef LINUX_BUILD
  #ifndef _GNU_SOURCE
    #ifdef _SYS_UCONTEXT_H
      #error "ucontext.h included without _GNU_SOURCE, please fix"
    #endif
    #define _GNU_SOURCE
    #undef _FEATURES_H
    #include <ucontext.h>
    #undef _GNU_SOURCE
  #else 
    #include <ucontext.h>
  #endif
#else
 #include <ucontext.h>
#endif 

#include "fdutils.h"
#include "j2c.h"


//extern "C" {
#include "engine.h"
#include "signalmanager.h"
#include "nativeScheduling.h"
#include "jvm_constants/throwables.h"
//}
#include "ffi.h"

#undef TRACE_THREADS
//#define TRACE_THREADS
#ifdef TRACE_THREADS
# define CTX(stuff...) ubprintf(stuff)
#else
# define CTX(stuff...) 
#endif

#ifdef CEXCEPTIONS
HEADER *cur_exc = 0;
int cur_exc_dom = 0;
#ifdef CEXCEPTIONS_COUNT
int n_exc_thrown = 0;
#endif
#endif

// typedef e_java_lang_Object Object;
// typedef e_ovm_core_domain_Oop Oop;
// typedef e_ovm_core_execution_ValueUnion ValueUnion;
typedef e_ovm_core_services_memory_VM_1Address VM_Address;

typedef e_s3_services_j2c_J2cCodeFragment J2cCodeFragment;

// extern e_Array<jint> cf_ranges;
// extern e_Array<J2cCodeFragment *> cf_frags;

static float bits2float(int x) {
  union {
    int i;
    float f;
  } u;

  u.i = x;
  return u.f;
}

static double bits2double(long long x) {
  union {
    long long l;
    struct {
     int x;
     int y;
    } swap;
    double d;
  } u;

  u.l = x;
#if defined(OVM_ARM) && !defined(WORDS_BIGENDIAN)
  int z=u.swap.x;
  u.swap.x=u.swap.y;
  u.swap.y=z;
#endif
  return u.d;
}

double _j2c_NaN;
double _j2c_nzero;
float _j2c_NaNf;
float _j2c_nzerof;

int **initializedBlueprints;
J2cContext *currentContext;
J2cContext *mainContext;
int process_argc;
char **process_argv;
extern void boot();

#ifdef OVM_XENOMAI
extern int exit_process_code;
extern RT_TASK main_ovm_task;
#endif

typedef struct pcRange {
  size_t start;
  size_t end;
} pcRange;

//extern e_Array<pcRange> j2c_method_ranges;
typedef struct e_Array_pcRange {
  e_java_lang_Object _parent_;
  int length;
  pcRange values[1];
} e_Array_pcRange;
extern e_Array_pcRange j2c_method_ranges;

//extern e_Array<e_s3_services_j2c_J2cCodeFragment *> j2c_method_pointers;
extern e_Array j2c_method_pointers;

static void compute_range_hints();
#ifdef PRECISE_BARRIER_PROF
extern e_Array<pcRange> barrier_prof_ranges;
#endif
static void init_barrier_prof();

int pgsize;

//inline void *operator new(size_t sz, void *p) { return p; }

ffi_type *codeToType(int code) {
  switch (code) {
  case 'L':
  case '[':
    return &ffi_type_pointer;
  case 'J':
    return &ffi_type_sint64;
  case 'I':
    return &ffi_type_sint32;
  case 'S':
    return &ffi_type_sint16;
  case 'C':
    return &ffi_type_uint16;
  case 'B':
  case 'Z':
    return &ffi_type_sint8;
  case 'D':
    return &ffi_type_double;
  case 'F':
    return &ffi_type_float;
  case 'V':
    return &ffi_type_void;
  default:
    assert(!"unknown type code");
  }
}

//jlong j2cInvoke(Code *code, InvocationMessage *args) {
jlong j2cInvoke(HEADER *codeh, HEADER *argsh) {
  Code *code = (Code *)codeh;
  InvocationMessage *args = (InvocationMessage *)argsh;
  
  J2cCodeFragment *cc = (J2cCodeFragment *) code;
  ffi_cif cif;

  // Should this be an assert, or a check that we perform even with 
  // ndebug?
  assert(HEADER_BLUEPRINT(cc) == ((e_s3_core_domain_S3Blueprint*)&bp_e_s3_services_j2c_J2cCodeFragment));
  
  if (cc->ovm_index == -1) {
    fprintf(stderr, "reflective call to `unreachable' method \"%.*s\"\n",
	    ((e_java_lang_String *)cc->ovm_cname)->ovm_count,
	    ((e_java_lang_String *)cc->ovm_cname)->ovm_data->values + ((e_java_lang_String *)cc->ovm_cname)->ovm_offset);
    abort();
  }
  int nargs = 1 + args->ovm_inArgs->length;
  ffi_type *at[nargs];
  ffi_type *rt = codeToType(cc->ovm_rtype);
  at[0] = codeToType('L');
  int i;
  for (i = 1; i < nargs; i++) {
    at[i] = codeToType(((e_ovm_core_execution_ValueUnion **)args->ovm_inArgs->values)[i-1]->ovm_tag);
  }
  
  ffi_status pstatus = ffi_prep_cif(&cif, FFI_DEFAULT_ABI, nargs, rt, at);
  assert(pstatus == FFI_OK);

  void (*fn) () = (void (*)()) j2c_method_ranges.values[cc->ovm_index].start;
  void *av[nargs];
  typedef union {
    jbyte b;
    jchar c;
    jshort s;
  } narrow;
  narrow av_narrow[nargs];

  av[0] = &args->ovm_receiver;
  for (i = 1; i < nargs; i++) {
    ValueUnion *ai = ((e_ovm_core_execution_ValueUnion **)args->ovm_inArgs->values)[i - 1];
    switch (ai->ovm_tag) {
    case 'L':
    case '[':
      av[i] = (void *) &ai->ovm_reference;
      break;
    case 'I':
    case 'F':
      av[i] = (void *) &ai->ovm_primitive;
      break;
    case 'D':
    case 'J':
      av[i] = &ai->ovm_widePrimitive;
      break;
    case 'Z':
    case 'B':
      av_narrow[i].b = ai->ovm_primitive;
      av[i] = (void *) &av_narrow[i].b;
      break;
    case 'S':
      av_narrow[i].s = ai->ovm_primitive;
      av[i] = (void *) &av_narrow[i].s;
      break;
    case 'C':
      av_narrow[i].c = ai->ovm_primitive;
      av[i] = (void *) &av_narrow[i].c;
      break;
    }
  }
  jvalue ret;
  HEADER *ex;
#ifdef COUNTER_EXCEPTIONS
  ffi_call(&cif, fn, &ret, av);
  if (CHECK_EXCEPTION()) {
      accurate::counterClearException();
      throwWildcardException(getCurrentException());
  }
  return ret.jv_jlong;

#elif defined CEXCEPTIONS

  cur_exc=(HEADER *)0;
  ffi_call(&cif, fn, &ret, av);
  
#if defined(PRECISE_SAFE_POINTS) && defined(PRECISE_THUNK)
  if (cur_exc && (cur_exc!=(HEADER *)&accurate::GCException_singleton)) {
#elif defined(PRECISE_SAFE_POINTS) && !defined(PRECISE_THUNK)
  if (accurate::gAuxStackHeight&ACC_EXCBIT)  {
#else
  if (cur_exc) {
#endif
    HEADER *caught_exception = cur_exc;
    cur_exc = 0;
#if defined(PRECISE_SAFE_POINTS) && !defined(PRECISE_THUNK)
    accurate::counterClearException();
#endif    

    /* this will just set the exception, the code
      will however continue executing at the following
      return statement

      for this to work, there must be no method call between 
      the call below and the return of this function */
    throwWildcardException(caught_exception);
    return 0;
  }
  return ret.jv_jlong;

#else
  int troubles=0;
  // FIXME: make this work with counter exceptions
  try { ffi_call(&cif, fn, &ret, av); }
  // Calling throwWildcardException from the handler
  // causes a rethrow of the original exception.
  // Jumping out of a catch block causes troubles on
  // GCC/ARM. So a flag is used instead.
  catch (HEADER *ex1) { ex = ex1; troubles=1; }

  if (troubles)
      throwWildcardException(ex);

  return ret.jv_jlong;
#endif
}

//jint j2cFail(...) {
jint j2cFail(void) {
  assert(!"abstract/boot-time/native method called\n");
  exit(-1);
}

sigset_t j2cNormalMask;

static inline void getMask(ucontext_t *ctx) {
  j2cNormalMask = ctx->uc_sigmask;

  stack_t stack;
  sigaltstack(0, &stack);
  if (stack.ss_flags & SS_ONSTACK)
    fprintf(stderr, "on stack %p of size %d (flags = %x)\n",
	    stack.ss_sp, stack.ss_size, stack.ss_flags);
}
#if defined(OVM_X86) && (defined(LINUX_BUILD) || defined(OSX_BUILD) || defined(NETBSD_BUILD) || defined(SOLARIS_BUILD))
/* Hopefully, this optimization will avoid stack fixup botches.  load
 * and divide instructions don't push arguments on the stack, so a
 * call that returns after such an instruction shouldn't push
 * arguments either.  (Maybe this must be used in combination with
 * -fno-defer-pop.)

/* ?? will everything work when this is uncommented ?
void generateThrowable_x86(int code)
  __attribute__((regparm(1)));
*/

void generateThrowable_x86(int code) {
  generateThrowable(code);
}

# if defined(LINUX_BUILD)
static void callGenerateThrowable(ucontext_t *state, int code)
{
  getMask(state);
  int *gregs = state->uc_mcontext.gregs;
  asm("movl 24(%2), %%ebp\n\t"	// "N"(4*REG_EBP) produces $24
      "movl 28(%2), %%esp\n\t"	// which is not allowed in loads
      //"pushl %0\n\t" already in %eax, can't be passed on stack
      "pushl %1\n\t"
      "jmp generateThrowable_x86"
      : :
      "a"(code), "c"(gregs[REG_EIP]+2), "d"(gregs),
      "D"(gregs[REG_EDI]), "S"(gregs[REG_ESI]), "b"(gregs[REG_EBX]));
}
# elif defined(NETBSD_BUILD)
static void callGenerateThrowable(ucontext_t *state, int code)
{
  getMask(state);
  int *gregs = state->uc_mcontext.__gregs;
  asm("movl 24(%2), %%ebp\n\t"	// "N"(4*REG_EBP) produces $24
      "movl 28(%2), %%esp\n\t"	// which is not allowed in loads
      //"pushl %0\n\t" already in %eax, can't be passed on stack
      "pushl %1\n\t"
      "jmp generateThrowable_x86"
      : :
      "a"(code), "c"(gregs[_REG_EIP]+2), "d"(gregs),
      "D"(gregs[_REG_EDI]), "S"(gregs[_REG_ESI]), "b"(gregs[_REG_EBX]));
}
# elif defined(SOLARIS_BUILD)
static void callGenerateThrowable(ucontext_t *state, int code)
{
  getMask(state);
  int *gregs = state->uc_mcontext.gregs;
  asm("movl 24(%2), %%ebp\n\t"	// "N"(4*REG_EBP) produces $24
      "movl 68(%2), %%esp\n\t"  // UESP rather than ESP on Solaris
      //"pushl %0\n\t" already in %eax, can't be passed on stack
      "pushl %1\n\t"
      "jmp generateThrowable_x86"
      : :
      "a"(code), "c"(gregs[EIP]+2), "d"(gregs),
      "D"(gregs[EDI]), "S"(gregs[ESI]), "b"(gregs[EBX]));
}
# elif defined(OSX_BUILD)
static void callGenerateThrowable(ucontext_t *state, int code) {
  getMask(state);
  asm("movl   4(%2), %%ebx\n\t"
      "movl  16(%2), %%edi\n\t"
      "movl  20(%2), %%esi\n\t"
      "movl  24(%2), %%ebp\n\t"
      "movl  28(%2), %%esp\n\t"
      "pushl %1\n\t"
      "jmp _generateThrowable_x86"
      : :
      "a"(code), "c"(UC_REG(state, UC_PC)+2), "d"(&state->uc_mcontext->ss));
}
# endif
#elif defined(OVM_PPC) && defined(OSX_BUILD)
static void callGenerateThrowable(ucontext_t *state, int code) {
  getMask(state);
  /* FIXME: this code assumes that callee-saved floating point
   * registers are unchanged. (they are fp14-fp31), and are available
   * in the mcontext.
   */
  ppc_thread_state *ctx = &state->uc_mcontext->ss;
  asm("mr   r3, %0\n\t"
      "mr   r4, %1\n\t"
      "mtlr %2\n\t"
      "mr   r1, %3\n\t"
      "lwz  r13,  0(r4)\n\t"
      "lwz  r14,  4(r4)\n\t"
      "lwz  r15,  8(r4)\n\t"
      "lwz  r16, 12(r4)\n\t"
      "lwz  r17, 16(r4)\n\t"
      "lwz  r18, 20(r4)\n\t"
      "lwz  r19, 24(r4)\n\t"
      "lwz  r20, 28(r4)\n\t"
      "lwz  r21, 32(r4)\n\t"
      "lwz  r22, 36(r4)\n\t"
      "lwz  r23, 40(r4)\n\t"
      "lwz  r24, 44(r4)\n\t"
      "lwz  r25, 48(r4)\n\t"
      "lwz  r26, 52(r4)\n\t"
      "lwz  r27, 56(r4)\n\t"
      "lwz  r28, 60(r4)\n\t"
      "lwz  r29, 64(r4)\n\t"
      "lwz  r30, 68(r4)\n\t"
      "lwz  r31, 72(r4)\n\t"
      "ba   _generateThrowable"
      : : "r"(code),  "r"(&ctx->r13), "r"(ctx->srr0 + 4), "r"(ctx->r1)
      : "r3", "r4");
}
#elif defined(OVM_PPC) && defined(LINUX_BUILD)
static void callGenerateThrowable(ucontext_t *state, int code) {
  getMask(state);
  /* This function differs from the Darwin version in terms of
   * register names, global symbol names (undersores), and ucontext_t
   * layout.
   */
  asm("mr   %%r3, %0\n\t"
      "mr   %%r4, %1\n\t"
      "mtlr %2\n\t"
      "mr   %%r1, %3\n\t"
      "lwz  %%r14,  0(%%r4)\n\t"
      "lwz  %%r15,  4(%%r4)\n\t"
      "lwz  %%r16,  8(%%r4)\n\t"
      "lwz  %%r17, 12(%%r4)\n\t"
      "lwz  %%r18, 16(%%r4)\n\t"
      "lwz  %%r19, 20(%%r4)\n\t"
      "lwz  %%r20, 24(%%r4)\n\t"
      "lwz  %%r21, 28(%%r4)\n\t"
      "lwz  %%r22, 32(%%r4)\n\t"
      "lwz  %%r23, 36(%%r4)\n\t"
      "lwz  %%r24, 40(%%r4)\n\t"
      "lwz  %%r25, 44(%%r4)\n\t"
      "lwz  %%r26, 48(%%r4)\n\t"
      "lwz  %%r27, 52(%%r4)\n\t"
      "lwz  %%r28, 56(%%r4)\n\t"
      "lwz  %%r29, 60(%%r4)\n\t"
      "lwz  %%r30, 64(%%r4)\n\t"
      "lwz  %%r31, 68(%%r4)\n\t"
      "lis  %%r5, generateThrowable@ha\n\t"
      "la   %%r4, generateThrowable@l(%%r5)\n\t"
      "mtctr %%r4\n\t" // may be too far away for ba
      "bctr"
      : :
      "r"(code),  "r"(&UC_REG(*state, UC_R14)),
      "r"(UC_REG(*state, UC_PC) + 4), "r"(UC_REG(*state, UC_R1))
      : "%r3", "%r4");
}
#elif defined(OVM_SPARC)
/*
Note:

the code below could be implemented in a much more elegant
manner by patching up the ucontext and simply returning,
so that control falls into generateThrowable, as follows:

void callGenerateThrowable(ucontext_t *state,int code) {
  int faultPC=state->uc_mcontext.gregs [REG_PC];
  state->uc_mcontext.gregs [REG_PC] = (int)generateThrowable;
  state->uc_mcontext.gregs [REG_nPC] = ((int)generateThrowable)+4;
  state->uc_mcontext.gregs [REG_O7] = faultPC-4;
  state->uc_mcontext.gregs [REG_O0] = code;
}

However, for consistency with the previous implementations
and to avoid possible dependencies from code that might
follow the call to callGenerateThrowable() in segvHandler,
the same pattern of registers restorations and call used
for the other architectures is used below.
*/

static void callGenerateThrowable(ucontext_t *state, int code) {
  getMask(state);
  asm(
      "ta   3\n\t"

// %l and %i registers are safely saved in the register windows,
// and will be restored when we return from this frame

// restore %g registers
      "ld   [%0+56],%%g1\n\t"    
      "ld   [%0+60],%%g2\n\t"  
      "ld   [%0+64],%%g3\n\t"
      "ld   [%0+68],%%g4\n\t"  
      "ld   [%0+72],%%g5\n\t"  
      "ld   [%0+76],%%g6\n\t"  
      "ld   [%0+80],%%g7\n\t"  

// restore %o registers into %i registers, they will become
// %o registers again in the code that caused the fault
// when we return from here
      "ld   [%0+84],%%i0\n\t"
      "ld   [%0+88],%%i1\n\t"
      "ld   [%0+92],%%i2\n\t"
      "ld   [%0+96],%%i3\n\t"
      "ld   [%0+100],%%i4\n\t"
      "ld   [%0+104],%%i5\n\t"
      "ld   [%0+108],%%i6\n\t" // now in fp (%i6), will end up in sp (%o6)

// %o7 in the frame of the faulting code *will* be
// overwritten, there's no way around that.
// Fingers crossed.
      "ld   [%0+44],%%i7\n\t" // pc -> ret address
      "sub  %%i7,4,%%i7\n\t"  // -4

      "mov  %1,%%o0\n\t"
      "call generateThrowable\n\t"
      "nop" // delay slot
       ::
       "r"(state),
       "r"(code)
       :
       "%o0","%g1","%g2","%g3",
       "%g4","%g5","%g6","%g7",
       "%i0","%i1","%i2","%i3",
       "%i4","%i5");
}
#elif defined(OVM_ARM) && defined(LINUX_BUILD)

/* Bizarrely, the structure that is returned by the Linux/ARM
   kernel as a third argument of a signal handler is *not*
   a ucontext_t, but rather the "struct ucontext" defined
   in the kernel headers, which is a totally different beast.

   Apparently this is due to a long-standing bug/inconsistency
   in glibc, which defines the wrong headers, or if seen in
   another way an inconsistency between the idea of ucontext
   that the kernel and glibc have. The issue has been discussed
   in this thread, for example:

   http://www.ussg.iu.edu/hypermail/linux/kernel/0304.2/0551.html

   More details here:

   http://lists.arm.linux.org.uk/pipermail/linux-arm-kernel/2003-April/014917.html

   Including the kernel headers would cause conficts with
   the standard signal.h, so I redefine manually some of the
   structures. The definitions below correspond to a 2.6.15
   kernel, but should be ok for any version since 2.1.79.

   Also note that fp regs are not touched by the signal
   handling infrastructure, so they are still set here
   as they were prior to the signal.
*/
#include <signal.h>

/* see: <asm/sigcontext.h> */
typedef struct asm_sigcontext {
  unsigned long trap_no;
  unsigned long error_code;
  unsigned long oldmask;
  unsigned long arm_r0;
  unsigned long arm_r1;
  unsigned long arm_r2;
  unsigned long arm_r3;
  unsigned long arm_r4;
  unsigned long arm_r5;
  unsigned long arm_r6;
  unsigned long arm_r7;
  unsigned long arm_r8;
  unsigned long arm_r9;
  unsigned long arm_r10;
  unsigned long arm_fp;
  unsigned long arm_ip;
  unsigned long arm_sp;
  unsigned long arm_lr;
  unsigned long arm_pc;
  unsigned long arm_cpsr;
  unsigned long fault_address;
} asm_sigcontext;

/* see: <asm/ucontext.h> */
typedef struct asm_ucontext {
  unsigned long         uc_flags;
  struct asm_ucontext  *uc_link;
  stack_t               uc_stack;
  struct asm_sigcontext uc_mcontext;
  sigset_t              uc_sigmask;
} asm_ucontext;


/* ARM_INSN_SIZE should be 4 for the standard instructioon
   set and 2 for the thumb. Must make --enable-thumb a parameter */

#if defined(ARM_THUMB)
#define ARM_INSN_SIZE (2)
#define ARM_THUMB_BIT 1
#define ARM_THUMB_BIT_STRING "1"
#else
#define ARM_INSN_SIZE (4)
#define ARM_THUMB_BIT 0
#define ARM_THUMB_BIT_STRING "0"
#endif

/* first arg is really a ptr to asm_ucontext */
static void callGenerateThrowable(ucontext_t *state, int code) {
  getMask(state);
  asm (
       "mov r2,%11\n\t"  /* lr */
       "mov r0,%10\n\t" /* arg to generateThrowable */
       "mov r1,%0\n\t"  /* used to load the rest */
       "mov lr,r2\n\t"
       "ldr r5,[r1,%2]\n\t"
       "ldr r6,[r1,%3]\n\t"
       "ldr r7,[r1,%4]\n\t"
#if defined(ARM_THUMB)
       "ldr r4,[r1,%5]\n\t"
       "mov r8,r4\n\t"
       "ldr r4,[r1,%6]\n\t"
       "mov r9,r4\n\t"
       "ldr r4,[r1,%7]\n\t"
       "mov r10,r4\n\t"
       "ldr r4,[r1,%8]\n\t"
       "mov fp,r4\n\t"
       "ldr r4,[r1,%9]\n\t"
       "mov sp,r4\n\t"
#else
       "ldr r8,[r1,%5]\n\t"
       "ldr r9,[r1,%6]\n\t"
       "ldr r10,[r1,%7]\n\t"
       "ldr fp,[r1,%8]\n\t"
       "ldr sp,[r1,%9]\n\t"
#endif
       "ldr r4,[r1,%1]\n\t"
       "ldr r1,.Lindirect_generateThrowable\n\t"
       "bx  r1\n"
       /* if thumb, set the lowest bit to 1, so that bx is happy */
".Lindirect_generateThrowable:\n\t.word generateThrowable + " ARM_THUMB_BIT_STRING
       ::
       "r"(state),
       "i"(offsetof(asm_ucontext,uc_mcontext.arm_r4)),
       "i"(offsetof(asm_ucontext,uc_mcontext.arm_r5)),
       "i"(offsetof(asm_ucontext,uc_mcontext.arm_r6)),
       "i"(offsetof(asm_ucontext,uc_mcontext.arm_r7)),
       "i"(offsetof(asm_ucontext,uc_mcontext.arm_r8)),
       "i"(offsetof(asm_ucontext,uc_mcontext.arm_r9)),
       "i"(offsetof(asm_ucontext,uc_mcontext.arm_r10)),
       "i"(offsetof(asm_ucontext,uc_mcontext.arm_fp)),
       "i"(offsetof(asm_ucontext,uc_mcontext.arm_sp)),
       "r"(code),
       "r"(((struct asm_ucontext*)state)->uc_mcontext.arm_pc+ARM_INSN_SIZE+ARM_THUMB_BIT)
       /* note: if we just move PC+4 or PC+2 in lr we lose the thumb bit.
	  We assume that the whole of Ovm is compiled either in thumb mode
	  or in regular ARM mode, so we set the flag as required in LR and
	  in the address of generateThrowable we jump to.
       */
       :
       "r0","r1","r2"
      );
}
#else
static void callGenerateThrowable(ucontext_t *state, int code) {
  assert(!"unabled to convert signal to exception");
}
#endif

static void segvHandler(int sig, siginfo_t * info, void *_ctx) {
#if 0
    ubprintf("segvHandler(%d, %p, %p)\n", sig, info, _ctx);
#endif
#if defined(PRECISE_SAFE_POINTS) && !defined(PRECISE_THUNK)
    if ((accurate::gAuxStackHeight+1)==0) {
	ubprintf("Aborting in segvHandler, since it was called during a stack walk!\n");
	abort();
    }
#endif
    ucontext_t *ctx = (ucontext_t *) _ctx;
    char *gaurdPage = ((char *) currentContext->stackBase) - pgsize;
    char *faultAddr = (char *) FAULT_ADDR(info, _ctx);
#if 0
    ubprintf("fault address = %p\n",faultAddr);
#endif
    if (faultAddr < (char*)pgsize) {
	if (0) ubprintf("throwing NPE from segv handler.\n");
	callGenerateThrowable(ctx, NULL_POINTER_EXCEPTION);
#ifdef STACK_OVERFLOW_DID_NOT_GENERATE_RECURSIVE_SEGV
    } else if (faultAddr >= gaurdPage) {
	ubprintf("throwing SOE from segv handler.\n");
	// Stack overflow currently triggers a recursive SEGV
	callGenerateThrowable(ctx, STACK_OVERFLOW_ERROR);
#endif
    } else {
      int reason = info->si_code;
      ubprintf("segvHandler: fault addr: %d\n", faultAddr);
      switch (reason) {
      case SEGV_MAPERR:   ubprintf(" reason: SEGV_MAPERR\n"); break;
      case SEGV_ACCERR:   ubprintf(" reason: SEGV_ACCERR\n"); break;
      default:            ubprintf(" reason: unknown(%d)\n", reason);
      }
      abort();
    }
}

void startContext(J2cContext *);

#if defined(PRECISE_SAFE_POINTS)
extern void userGC(accurate::AuxStack *);
#endif

static void reallyStartTheDamnedThread() {
  J2cContext *ctx = currentContext;
  Code *code=ctx->code;
  InvocationMessage *args=ctx->args;
  ctx->code=0;
  ctx->args=0;
#ifdef PRECISE_PTRSTACK
  ctx->gcTop=ctx->ptrStack;
#endif
  //assertAddressValid(code);
  //assertAddressValid(args);
  j2cInvoke((HEADER *)code, (HEADER *)args);
}

void startContext(J2cContext *ctx) {
#ifndef THREAD_UCONTEXT
  // Initially wake up on our own stack
  if (!setjmp(ctx->buf))
    // Return to newNativeContext on the parent thread's stack
    longjmp(currentContext->buf, 1);
#endif
  CTX("%p starting\n", ctx);
  currentContext = ctx;
  ctx->bottomFrame = getCurrentActivation(0);
#if defined(PRECISE_SAFE_POINTS)
  ctx->localState.run(reallyStartTheDamnedThread, userGC);
#else
  reallyStartTheDamnedThread();
#endif

  write(2, "unexpected return from thread's initial method", 46);
  abort();
}

#define MEM2CTX(mem) 							\
  ((void *) (((char *) mem) + CONTEXT_SIZE + pgsize - ((sizeof(J2cContext)+7)&~7) ))

#define CTX2MEM(ctx) 							\
  (((char *) ctx) - CONTEXT_SIZE - pgsize + ((sizeof(J2cContext)+7)&~7) )

inline J2cContext* J2cContext_init(J2cContext *jc, Context *oc) {
#ifdef PRECISE_HENDERSON
    jc->gcTop = 0;
#elif defined(PRECISE_PTRSTACK)
    jc->gcTop = 0;
#elif defined(PRECISE_SAFE_POINTS)
    jc->shouldWalkStack = 0;
#endif
    jc->ovmContext = oc;  
    return jc;  
}


//int newNativeContext(Context *appCtx) {
int newNativeContext(HEADER *appCtxh) {
  Context *appCtx = (Context *)appCtxh;
  
  // allocate stack-gaurd page
  void *mem = mmap(0, CONTEXT_SIZE + pgsize,
		   PROT_READ|PROT_WRITE|PROT_EXEC,
		   MAP_PRIVATE|MAP_ANON, -1, 0);

  // Place context at the end of the allocation,
  // the macros will keep it 8-byte aligned
  J2cContext *ctx;
  void *_ctx = MEM2CTX(mem);
  
//  ctx = new(_ctx) J2cContext(appCtx);
  ctx = J2cContext_init(_ctx,appCtx);

  // Stack starts just below the context and ends just above the gaurd page
  ctx->stackBase = (void *) (((size_t) ctx) - 8 );
  // keep stack base 8-byte aligned as well

  ctx->stackTop = (void *) (((char *) mem) + pgsize);
  CTX("thread %p has stack %p - %p\n", ctx, ctx->stackTop, ctx->stackBase);
  mprotect((void *) mem, pgsize, PROT_NONE);

#ifdef THREAD_UCONTEXT
  getcontext(&ctx->uc);
  // in which direction does the stack grow?  What is expected here?
  ctx->uc.uc_stack.ss_sp = ctx->stackTop;
  ctx->uc.uc_stack.ss_flags = 0;
  ctx->uc.uc_stack.ss_size = CONTEXT_SIZE - sizeof(*ctx);
  ctx->uc.uc_link = 0;
  makecontext(&ctx->uc, (void (*)()) startContext, 1, ctx);
#else
#define START(callee_save1, callee_save2, start_asm) {			\
    /* setjmp is going to return 1 long after this frame has died.	\
     * Only use one variable, ctx1, stored in a callee-save register.	\
     */									\
    register void *stackBase asm (#callee_save1) = ctx->stackBase;	\
    register J2cContext *ctx1 asm (#callee_save2) = ctx;		\
									\
    if (!setjmp(currentContext->buf))					\
      {									\
	/* FIXME: need underscore? factor into x86-specific header? */	\
	asm(start_asm  : : "r"(stackBase), "r"(ctx1));			\
	write(2, "unexpected return from startContext\n", 36);		\
	abort();							\
      }									\
  }
    
# if defined(OVM_X86)
#  ifdef ASM_NEEDS_UNDERSCORE
#    define startContextSym "_startContext"
#   else
#     define startContextSym "startContext"
#  endif
  START(edi, esi,
	"andl $-16, %0\n"
	"movl %0, %%esp\n"
	"movl %0, %%ebp\n"
	"subl $12, %%esp\n"
	"pushl %1\n"
	"call " startContextSym "\n"
	);
# elif defined(OVM_PPC)
  /*
   * Here's what I know:
   * r3-r12 are argument registers
   * r13    is the first callee-save register
   * r1     is the stack pointer
   *
   * bl places the return address in a special reg `lr'
   * gcc's prologue saves it at 8(r1) (in the caller's frame), and
   * stores the caller's stack pointer at 0(r1) (in the callee's
   * frame).
   *
   * We need to allocate some space in the bottom frame. We need
   * enough space to hold the frame information and spill the three
   * arguments to write.  I guess that would be 5 words, but it should
   * be rounded up to 8 to satisfy any alignment requirements.
   * (It looks like gcc aligns r1 to quadword boundaries.)
   *
   * I don't understand the PIC magic to get the function's base
   * address into r31
   */
#  ifdef OSX_BUILD
  START(r16, r15,
	"addi r1, %0, -32\n"
	"stw  r1, 0(r1)\n"
	"mr   r3, %1\n" 
	"bl   _startContext");
#  else
  /* The thread-launching asm for ppc-linux seems to work, but unless
   * we use swapcontext, certain library functions such as malloc can
   * only be called from the main thread.
   */
  START(%r16, %r15,
	"addi %%r1, %0, -32\n"
	"stw  %%r1, 0(%%r1)\n"
	"mr   %%r3, %1\n" 
	"bl   startContext");
#  endif
# elif defined (OVM_ARM)
#  if defined(ARM_THUMB)
    START(r4,r5,
	  "mov   sp,%0\n"
	  "mov   r0,%1\n"
	  "push  {r7, lr}\n"
	  "sub   sp, sp, #32\n"
	  "add   r7, sp, #0\n"
	  "bl    startContext");
#  else
    START(r4,r5,
	  "mov   sp,%0\n"
	  "mov   r0,%1\n"
          "mov   ip, sp\n"
	  "stmfd sp!, {fp, ip, lr, pc}\n"
	  "sub   fp, ip, #4\n"
	  "sub   sp, sp, #32\n"
	  "bl    startContext");
#  endif
# elif defined (OVM_SPARC) && defined(LINUX_BUILD)

// The final "mov" is in the delay slot.
// 64 bytes must always be available to save in and local regs.
// as I'm moving the stack, and that interferes with the register
// windows flushing, I also save explicitly ret addr and fp.
// That *should* be enough for stack traversal. Even though
// it is not really necessary, I save all local and in regs

// ta 3 is ta ST_FLUSH_WINDOWS

  START(%l0,%l1,
	  "ta 3\n"

	  "sub  %0,128,%0\n"
	  "mov  %0,%%sp\n"

	  "st   %%l0,[%%sp+ 0]\n"
	  "st   %%l1,[%%sp+ 4]\n"
	  "st   %%l2,[%%sp+ 8]\n"
	  "st   %%l3,[%%sp+12]\n"

	  "st   %%l4,[%%sp+16]\n"
	  "st   %%l5,[%%sp+20]\n"
	  "st   %%l6,[%%sp+24]\n"
	  "st   %%l7,[%%sp+28]\n"

	  "st   %%i0,[%%sp+32]\n"
	  "st   %%i1,[%%sp+36]\n"
	  "st   %%i2,[%%sp+40]\n"
	  "st   %%i3,[%%sp+44]\n"

	  "st   %%i4,[%%sp+48]\n"
	  "st   %%i5,[%%sp+52]\n"
	  "st   %%i6,[%%sp+56]\n"
	  "st   %%i7,[%%sp+60]\n"

	  "call startContext\n"
	  "mov  %1,%%o0")

# elif defined (OVM_SPARC) && defined(SOLARIS_BUILD)

// The final "mov" is in the delay slot.
// 64 bytes must always be available to save in and local regs.
// as I'm moving the stack, and that interferes with the register
// windows flushing, I also save explicitly ret addr and fp.
// That *should* be enough for stack traversal. Even though
// it is not really necessary, I save all local and in regs,
// just in case.
// Some more space is reserved for saving return address and
// as home for parameters of called functions. 128 bytes
// are plenty

// ta 3 is ta ST_FLUSH_WINDOWS

  START(%l0,%l1,
	  "ta 3\n"

	  "sub  %0,128,%0\n"
	  "mov  %0,%%sp\n"

	  "st   %%l0,[%%sp+ 0]\n"
	  "st   %%l1,[%%sp+ 4]\n"
	  "st   %%l2,[%%sp+ 8]\n"
	  "st   %%l3,[%%sp+12]\n"

	  "st   %%l4,[%%sp+16]\n"
	  "st   %%l5,[%%sp+20]\n"
	  "st   %%l6,[%%sp+24]\n"
	  "st   %%l7,[%%sp+28]\n"

	  "st   %%i0,[%%sp+32]\n"
	  "st   %%i1,[%%sp+36]\n"
	  "st   %%i2,[%%sp+40]\n"
	  "st   %%i3,[%%sp+44]\n"

	  "st   %%i4,[%%sp+48]\n"
	  "st   %%i5,[%%sp+52]\n"
	  "st   %%i6,[%%sp+56]\n"
	  "st   %%i7,[%%sp+60]\n"

	  "call startContext\n"
	  "mov  %1,%%o0")

# else
#  error "Don't know how to start threads on this system"
# endif
#endif
  return (int) ctx;
}

void destroyNativeContext(int _ctx) {
  J2cContext *ctx = (J2cContext *) _ctx;
  if (ctx != mainContext)
    munmap(CTX2MEM(ctx), CONTEXT_SIZE + pgsize);
}

void run(int processorHandle, int _ctx) {
 top: {
 
  J2cContext *ctx = (J2cContext *) _ctx;
  assert(processorHandle == 0);
  J2cContext *me = currentContext;

  if (ctx == me) {
    CTX("skipping switch %p -> %p\n", ctx, ctx);
  } else {
    // topFrame is run(), top java frame's PC available here
    me->topFrame = getCurrentActivation(0);


    CTX("swap %p -> %p\n", me, ctx);

#ifdef THREAD_SETJMP
    if (setjmp(me->buf)) {
      currentContext = me;
    } else {
      longjmp(ctx->buf, 1);
    }
#else
    swapcontext(&me->uc, &ctx->uc);
    currentContext = me;
#endif

    CTX("%p waking up\n", me);
  }
  }

#if defined(PRECISE_SAFE_POINTS)
  me->localState.makeCurrent();
  if (me->shouldWalkStack) {
      CTX("should walk stack!\n");
      me->shouldWalkStack = 0;
      me->localState.gc();

#if defined(CEXCEPTIONS)
#if defined(PRECISE_THUNK)
      if (cur_exc) {
        CTX("gc threw an exception.\n");
        return ;
      }
#else
      if (accurate::gAuxStackHeight>=ACC_SPECIAL_THRESHOLD) {
        CTX("gc special return (exception)");
        return ;
      }
#endif

#else /* not CEXCEPTIONS */
      CHCKRET();
#endif      

      _ctx = nextThreadToWalk();
      goto top;
  } else {
      CTX("not walking any stacks today.\n");
  }
#endif
}

//jint makeActivation(jint _ctx, Code *cf, InvocationMessage *invocation)
jint makeActivation(jint _ctx, HEADER *cfh, HEADER *invocationh)
{
  Code *cf = (Code *)cfh;
  InvocationMessage *invocation = (InvocationMessage *)invocationh;
  
  J2cContext  *ctx = (J2cContext *) _ctx;
  /*
  if (ctx->code)
    {
      fprintf(stderr, "makeActivation called after thread start\n");
      abort();
    }
  */
  CTX("in makeActivation\n");
  //assertAddressValid(cf);
  //assertAddressValid(invocation);
  ctx->code = cf;
  ctx->args = invocation;
  return 0;
}

void launchContext(void *ctx) {
  assert(!"not supported");
}

#ifdef BOOTBASE
static ImageFormat *image = (ImageFormat *) BOOTBASE;
#else
static ImageFormat *image;
#endif

void *getImageBaseAddress() {
  return (void*) ((char*)image->data + 4); /* first word is null */
}

void *getImageEndAddress() {
  int off = image->usedMemory - 4; /* last word is magic number */
  return (void *) ((char *)image->data + off);
}

const int signalsReservedByExecEngine[] = {
  /* add comma-separated list of signals that J2c uses for aborting or
   * regular execution here, but before the 0.  see interpreter/main.c
   * for an example. */
  SIGSEGV, SIGBUS,
#ifdef PRECISE_SAFE_POINTS
  PRECISE_RESERVED_SIGNAL,
#endif
  0
};

jlong getNow() {
// This code is similar to getCurrentTime()
// in src/native/common/native_helpers.c
#if _POSIX_TIMERS > 0
    struct timespec tm;
    int status = clock_gettime(CLOCK_REALTIME, &tm);
    jlong nanos_per_sec = 1000L * 1000 * 1000;
    jlong now = tm.tv_sec * nanos_per_sec + tm.tv_nsec;
    if (status != 0) {
        ASSERT(status == -1 && errno == EINVAL, "unexpected error from clock_getttime");
        return status;
    }
    return now;
#else
     struct timeval now;
     jlong micros_per_sec = 1000000;
     jlong nowNanos = 0;
     gettimeofday(&now, NULL);
/*      printf("Using gettimeofday()\n");  */
     nowNanos = (now.tv_usec + now.tv_sec * micros_per_sec) * 1000;
     return nowNanos;
#endif
}

#if defined(SOLARIS_BUILD)
// on solaris, mmap totally ignores the supplied address,
// unless MAP_FIXED is used. OTOH, MAP_FIXED destroys
// previous existing mappings. The very convoluted code
// below detects whether the address space requested is
// actually free, recycling the code at
// http://www.winehq.com/hypermail/wine-devel/2000/11/0202.html

#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <assert.h>
#include <alloca.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <procfs.h>

static int is_mapped_test (uintptr_t vaddr, size_t size,
			   const prmap_t *asmap, int n)
{
  int i = 0, j = n;

  while (i < j)
  {
    int m = (i + j) / 2;
    const prmap_t *o = &asmap[m];

    if ((uintptr_t) o->pr_vaddr >= vaddr + size)
      j = m;
    else if ((uintptr_t) o->pr_vaddr + o->pr_size <= vaddr)
      i = m + 1;
    else
      return 1;
  }

  return 0;
}


static void *safe_mmap (void *addr, size_t len, int prot, int flags,
			int fildes, off_t off)
{
  if (flags & MAP_FIXED)
    return mmap (addr, len, prot, flags, fildes, off);
  else
  {
    int stat = 0;
    pid_t pid;
    int fd;
    struct stat sb;
    prmap_t *asmap;
    void *actual_addr;

    fd = open ("/proc/self/rmap", O_RDONLY);
    assert (fd != -1);
    if ((pid = vfork ()) == -1)
    {
      perror ("is_mapped: vfork");
      abort ();
    }
    else if (pid == 0)
    {
      fstat (fd, &sb);
      asmap = (prmap_t *) alloca (sb.st_size);
      read (fd, asmap, sb.st_size);
      if (is_mapped_test ((uintptr_t) addr, len, asmap,
			  sb.st_size / sizeof (prmap_t)))
	_exit (EADDRINUSE);
      else if ((actual_addr = mmap (addr, len, prot, flags | MAP_FIXED,
				    fildes, off)) == (void *) -1)
	_exit (errno);
      else if (actual_addr != addr)
      {
	munmap (actual_addr, len);
	kill (getpid (), SIGKILL);
      }
      else
      {
	_exit (0);
      }
    }
    else if (waitpid (pid,  &stat, WNOHANG) != pid)
    {
      perror ("is_mapped: waitpid");
      abort ();
    }
    close (fd);
    if (!WIFEXITED (stat))
      return mmap (addr, len, prot, flags, fildes, off);
    else if (WEXITSTATUS (stat) == 0)
      return addr;
    else if (WEXITSTATUS (stat) == EADDRINUSE)
      return mmap (addr, len, prot, flags, fildes, off);
    else
    {
      errno = WEXITSTATUS (stat);
      return (void *) -1;
    }
  }
}
#endif

#ifdef OVM_XENOMAI
RT_TASK task;

int real_argc;
char **real_argv;

void task_body (void *cookie) {

    /* Ask Xenomai to warn us upon switches to secondary mode. */
//    rt_task_set_mode(0, T_WARNSW, NULL);

    /* A real-time task always starts in primary mode. */

//  fprintf(stderr, "About to call real main from OVM Xenomai task...\n");
  fflush(stderr);
    
    real_main(real_argc,real_argv);
    /* the real main in OVM does not return */
}

void warn_upon_switch(int sig __attribute__((unused))) {

    void *bt[32];
    int nentries;

    /* Dump a backtrace of the frame which caused the switch to
       secondary mode: */
    nentries = backtrace(bt,sizeof(bt) / sizeof(bt[0]));
    backtrace_symbols_fd(bt,nentries,fileno(stdout));
    
    /*
    This only makes sense when the switches are rare... very rare
    
    fprintf(stderr,"Switching back to primary mode.\n");
    rt_task_set_mode(0, T_PRIMARY, NULL);
    */
}

int main(int argc, char **argv) {

  int err;

  real_argc = argc;
  real_argv = argv;
  
  /* this should be called automatically from a library constructor, but with current
     linking in OVM, it is not */
//  __rt_print_init();
  
    signal(SIGXCPU, warn_upon_switch);
    mlockall(MCL_CURRENT|MCL_FUTURE);
  
    rt_print_auto_init(1);

    err = rt_print_init(4096, "ovmtask");
    if (err) {
      fprintf(stderr,"Failed to alllocate RT print buffer, code %d (%s)\n", err, strerror(-err) );
      exit(2);
    }
  
//  err = rt_task_create(&task,"ovmtask",0,11,T_FPU|T_JOINABLE);  

//  this normally works, but in case there is no blocking calls (i.e. a
//  simple serial port polling application), it locks up the system via
//  starving Linux

//  err = rt_task_create(&task,"ovmtask",0,11,T_FPU);  

  // for a reason unclear to me, 0 does not starve Linux
  
  err = rt_task_create(&task,"ovmtask",0,0,T_FPU);  

  if (err) {
    fprintf(stderr,"Failed to create Xenomai OVM task, code %d (%s)\n", err, strerror(-err) );
    exit(2);
  }
  
  // this task will only sleep and wait for the end ; thus make sure it gets to the sleep before OVM ends
  err = rt_task_shadow( &main_ovm_task, NULL, 99, 0 );

  if (err) {
    fprintf(stderr,"Shadowing bootstrap task failed, code %d (%s)\n", err, strerror(-err));
    exit(2);
  }  


//  fprintf(stderr, "Running Xenomai OVM task...\n");
  fflush(stderr);
  
  err = rt_task_start(&task,&task_body,NULL);

  if (err) {
    fprintf(stderr,"Running Xenomai OVM task failed, code %d (%s)\n", err, strerror(-err) );
    rt_print_cleanup();    
    exit(2);
  }

  err = rt_task_suspend( &main_ovm_task );
  if (err) {
    fprintf(stderr,"Bootstrap task failed to suspend/resume, code %d (%s)\n", err, strerror(-err));
  }
  
  if (exit_process_code==-1) {
    fprintf(stderr, "BUG: Bootstrap task was resumed too early, switching to active waiting !\n");
  }
  
  /* very nasty workaround */
  while(exit_process_code==-211) {
    sleep(1);
  }

/*  
  err =  rt_task_join(&task);

  if (err) {
    fprintf(stderr,"Joining Xenomai OVM task failed, code %d\n",err);
    exit(2);
  }
*/

/*
  main_ovm_task = rt_task_self();
  err = rt_task_suspend(main_ovm_task);
  if (err) {
    perror("Failed to suspend bootstrap OVM task, rt_task_suspend");
    fprintf(stderr, "rt_task_suspend returned %d.\n", err);
    exit(2);
  }  
*/  
  fprintf(stderr, "Xenomai task finished.\n");
  return exit_process_code;
}


int real_main(int argc, char **argv) {

#else
// no Xenomai
int main(int argc, char **argv) {
#endif

  
#if defined(OVM_X86) && (defined(LINUX_BUILD))

  // set IO privilege level on x86 to allow port access, if possible
  int res = iopl(3);
  
  if (!res) {
//    fprintf(stderr, "I/O privilege level set to 3\n");
  } else {
    fprintf(stderr, "WARNING: Could not set I/O privilege level - port access will cause SEGFAULT.\n");
  }

#endif

// cheap workaround to a minor qemu deficiency
// remove the ifndef when debug is complete.         FIXME
#if 0
  long long now=getNow();
  printf("Initial nanoseconds from the Epoch: %lld  (0x%016llX)\n",now,now);
  fflush(stdout);
#endif

  _j2c_NaN =   bits2double(0x7ff8000000000000LL);
  _j2c_nzero = bits2double(0x8000000000000000LL);
  _j2c_NaNf =   bits2float(0x7fc00000);
  _j2c_nzerof = bits2float(0x80000000);
  
  pgsize = getpagesize();

  process_argc = argc;
  process_argv = argv;

#ifndef BOOTBASE
  if (process_argc < 2) {
    fprintf(stderr, "usage: %s <image> arguments\n", argv[0]);
    exit(1);
  }
  int fd = open(process_argv[1], O_RDONLY);
  if (fd < 0) {
    perror("opening image file");
  }
  process_argv++;
  process_argc--;
  ImageFormat tmpImage;
  read(fd, &tmpImage, sizeof(tmpImage));
  lseek(fd, 0, SEEK_SET);
  int length = lseek(fd, 0, SEEK_END);
  /* Round up request to a multiple of page size.  The remainder of
   * the last page will be zero-filled.  Not rounding up may cause
   * the OS to round down.
   */
  length = (length + pgsize - 1) & ~(pgsize - 1);


#if defined(SOLARIS_BUILD)
  void *_image = safe_mmap((void *) tmpImage.baseAddress,
		      length,
		      PROT_READ|PROT_WRITE|PROT_EXEC,
		      MAP_PRIVATE,
		      fd,
		      0);
#else

  void *_image = mmap((void *) tmpImage.baseAddress,
		      length,
		      PROT_READ|PROT_WRITE|PROT_EXEC,
		      MAP_PRIVATE,
		      fd,
		      0);
		      
#endif
  if (_image != (void *) tmpImage.baseAddress) {
    perror("could not map image to base address");
    exit(1);
  }

  image = (ImageFormat *) _image;
  if (image->mainObject != &mainObject
      || image->coreServicesAccess != &coreServicesAccess
      || image->bootContext != &bootContext) {
    fputs("Image version does not agree with compiled code\n", stderr);
    exit(-1);
  }
#endif
  if (process_argc > 1 && !strcmp(process_argv[1], "-dump-method-sizes")) {
    int nRanges = (j2c_method_pointers.length);
    pcRange *range = &j2c_method_ranges.values[0];
    e_s3_services_j2c_J2cCodeFragment **method =
      &(((e_s3_services_j2c_J2cCodeFragment **)j2c_method_pointers.values)[0]);

    int i;
    for (i = 0; i < nRanges; i++)
      printf("%5d\t%s\n",
	     (range[i].end - range[i].start + 31) & -32,
	     ( ((e_java_lang_String *)method[i]->ovm_cname)->ovm_data->values
	      + ((e_java_lang_String *)method[i]->ovm_cname)->ovm_offset));
    exit(0);
  }
	      
  initializedBlueprints = (int **) malloc(3 * sizeof(int));
  initializedBlueprints[1] =
    (int *) malloc(((e_s3_core_domain_S3TypeContext *) ( (( e_s3_core_domain_S3Domain*)&u1)->ovm_context_1))->ovm_blueprintCount/8);
  memset(initializedBlueprints[1], 0, ((e_s3_core_domain_S3TypeContext *)( (( e_s3_core_domain_S3Domain *)&u1)->ovm_context_1))->ovm_blueprintCount/8);
  initializedBlueprints[2] =
    (int *) malloc(((e_s3_core_domain_S3TypeContext *)u1.ovm_appContext_1)->ovm_blueprintCount/8);
  memset(initializedBlueprints[2], 0,
	 ((e_s3_core_domain_S3TypeContext *)u1.ovm_appContext_1)->ovm_blueprintCount/8);
  EXIT_ERRNO(initSignalHandling(),"initSignalHandling");

  compute_range_hints();

  init_barrier_prof();
  
#if defined(PRECISE_SAFE_POINTS) && !defined(PRECISE_THUNK)
  accurate::initCounterSignalEvent();
#endif
  
// not for Xenomai  
//  initializeNativeScheduling();

/*
not needed anymore - SW barrier, SW nullchecks
  struct sigaction act;
  act.sa_sigaction = segvHandler;
  sigfillset(&act.sa_mask);
  act.sa_flags = SA_SIGINFO;
  if (sigaction(SIGSEGV, &act, 0) < 0)
    perror("installing SEGV handler");
  if (sigaction(SIGBUS, &act, 0) < 0)
    perror("installing BUS handler");
*/
  static contextInitialized = 0;
  static J2cContext c;
  if (!contextInitialized) {
    J2cContext_init(&c,(e_ovm_core_execution_Context *)&bootContext);
    contextInitialized = 1;
  }
  
  currentContext = mainContext = &c;
  mainContext->bottomFrame = getCurrentActivation(0);
  mainContext->stackBase = (void *) getCurrentActivation(0);
  ((e_ovm_core_execution_Context *)&bootContext)->ovm_nativeContextHandle_1 = (jint) mainContext;

#ifdef PRECISE_PTRSTACK
  c.gcTop=c.ptrStack;
#endif

#ifdef OVM_XENOMAI
// enable this when catching switches to secondary domain  
//  rt_task_set_mode(0, T_PRIMARY|T_WARNSW, NULL);

  rt_task_set_mode(0, T_PRIMARY, NULL);
#endif  
  
#if defined(PRECISE_SAFE_POINTS)
  mainContext->localState.run(boot, userGC);
#else
  boot();
#endif
}

inline static int is_array(S3Blueprint* bpt) {
    return bpt->ovm_arrayDepth_1 > 0;
}

static S3Blueprint* strip_array_blueprint(S3Blueprint* bpt, int howManyTimes) {
    int i;
    assert(howManyTimes <= bpt->ovm_arrayDepth_1);
    for (i = 0; i < howManyTimes; i++) {
	bpt = (S3Blueprint *)
	  ((e_s3_core_domain_S3Blueprint_Array*)bpt)->ovm_componentBlueprint_1;
    }
    return bpt;
}

//jboolean is_subtype_of(S3Blueprint* client, S3Blueprint* provider) {
jboolean is_subtype_of(HEADER* clienth, HEADER* providerh) {

    S3Blueprint *client = (S3Blueprint*)clienth;
    S3Blueprint *provider = (S3Blueprint*)providerh;

    if (client == provider) { /* equality should work for primitives */
	return 1;
    }
    assert(client->ovm_typeDisplay_1);

    // if client is primitive, typeDisplay_.length == 0
    if (provider->ovm_typeBucket_1 >= client->ovm_typeDisplay_1->length)
      return 0;
    
    if (client->ovm_typeDisplay_1->values[provider->ovm_typeBucket_1] 
	== provider->ovm_typeBucketID_1) {
	return 1;
    } 
    /*	printf("failed display[%d] = %d != %d\n",
 	provider->typeBucket_,
 	client->typeDisplay_->values[provider->typeBucket_]
 	provider->typeBucketID_);
    */
    
    if (!is_array(provider)) {
	return 0; /* the test was subsumed */ 
    } else { /* provider is an array, test doesn't matter */
	if (!is_array(client)) {
	    return 0;
	} else {
	    int cdepth = client->ovm_arrayDepth_1;
	    int pdepth = provider->ovm_arrayDepth_1;
	    if (pdepth > cdepth) { 
		return 0;
	    } else { /* pdepth <= cdepth */
		return is_subtype_of((HEADER *)strip_array_blueprint(client, pdepth),
				     (HEADER *)strip_array_blueprint(provider, pdepth));
	    }
	}
    } 
}

void did_init(jint ctx, jint bp) {
  // Repeatedly do CSA calls during startup so that the test in
  // needs_init will be fast later on.
  if (!((CSA *) ( ((e_s3_core_domain_S3Domain*)&u1)->ovm_myCSA))->ovm_classInitializationEnabled)
    return;
  int w = bp >> 5;
  int b = 1 << (bp & 31);
  initializedBlueprints[ctx][w] |= b;
}

int stacktrace(NativeContext *_) {
  abort();
  return 0;
}

static size_t range_offset;	// first java bytecode pc
static size_t range_size;	// last java bytecode pc - range_offset

/*
 * range_hints is essentially a hashtable that tells us the first
 * method that a pc may be in.  RANGE_HASH(pc - range_offset) gives us
 * the higher-order bits of the difference between pc and the start of
 * the first java method, and
 * range_hints[RANGE_HASH(pc - range_offset) give us the index of the
 * first java method that shares higher-order bits with pc.
 *
 * The number 7 was chosen based on `gen-ovm -opt=run' on an x86.
 * In a few places there where long sequences of methods 16 bytes
 * apart, and it was more common to see methods 32 or 48 bytes apart.
 * shifting 7 means that we should never have to search more than 8 or
 * 9 entries in the range table, and the average will be very close to
 * 1.  It uses quite a bit of memory: 3% of range_size.
 */
#define RANGE_HASH(X) ((X) >> 7)
static int *range_hints;

static void compute_range_hints() {
  range_offset = j2c_method_ranges.values[0].start;
  range_size = j2c_method_ranges.values[j2c_method_pointers.length-1].end
		- range_offset;

  int n_hints = RANGE_HASH(range_size) + 1;
  //range_hints = new int[n_hints];
  range_hints = (int *)malloc(n_hints*sizeof(int));
#if 0
  fprintf(stderr, "allocating %d range hints\n", n_hints);
#endif

  int ridx = 0;
  int hidx;
  for (hidx = 0; hidx < n_hints; hidx++) {
    while (ridx < j2c_method_pointers.length 
	   && RANGE_HASH(j2c_method_ranges.values[ridx].end - range_offset) < hidx)
      ridx++;
    range_hints[hidx] = ridx;
  }
}

#ifdef PRECISE_BARRIER_PROF
static size_t bp_range_offset;
static size_t bp_range_size;

static int *bp_range_hints;

static unsigned long long bp_total=0;
static unsigned long long bp_inbarrier=0;

// FIXME: this currently is only meant for x86
void bp_timer_handler(jint sig, siginfo_t *info, void *ctx) {
    bp_total++;
    
    struct ucontext *uctx=(struct ucontext*)ctx;
    size_t pc = (size_t) uctx->uc_mcontext.gregs[REG_EIP];
    size_t delta = pc - bp_range_offset;
    if (delta >= bp_range_size) {
	return;
    }
    int idx = bp_range_hints[RANGE_HASH(delta)];
    while (barrier_prof_ranges.values[idx].end <= pc) {
	idx++;
    }
    if (barrier_prof_ranges.values[idx].start <= pc) {
	bp_inbarrier++;
    }
}

void bp_exit_handler() {
    fSBE(stderr,1);
    fprintf(stderr,
	    "Barrier Prof: total = %llu, in barrier = %llu\n",
	    bp_total,
	    bp_inbarrier);
}
#endif

static void init_barrier_prof() {
#ifdef PRECISE_BARRIER_PROF
#warning "Doing barrier profiling!"
  bp_range_offset = barrier_prof_ranges.values[0].start;
  bp_range_size = (barrier_prof_ranges.values[barrier_prof_ranges.length-1].end
		   - bp_range_offset);

  int n_hints = RANGE_HASH(bp_range_size) + 1;
  // bp_range_hints = new int[n_hints];
  bp_range_hints = (int *)malloc(n_hints*sizeof(int));
#if 1
  fprintf(stderr, "allocating %d barrier prof range hints\n", n_hints);
#endif

  int ridx = 0;
  for (int hidx = 0; hidx < n_hints; hidx++) {
    while (ridx < barrier_prof_ranges.length 
	   && RANGE_HASH(barrier_prof_ranges.values[ridx].end
			 - bp_range_offset) < hidx)
      ridx++;
    bp_range_hints[hidx] = ridx;
  }
  
  // register SIGALRM signal handler.  we rely on the code in timer.c to
  // register a sufficiently high frequency timer.  It think that it currently
  // uses a 100Hz timer, which should be good enough.
// this is done in timer.c again - maybe it was for a reason here, too, but
// I disabled it because of Xenomai  
//  ABORT_ERRNO(registerSignalHandler(SIGALRM,bp_timer_handler),
//	      "registerSignalHandler");
//
  
  atexit(bp_exit_handler);
#endif
}
  
void* j2c_getCode(void *_pc) {
  size_t pc = (size_t) _pc;
  size_t delta = pc - range_offset;
  if (delta >= range_size) {
#if 0
    fprintf(stderr, "getCode(%p) => <null>, out of range\n", _pc);
#endif
    return 0;
  }
  int idx = range_hints[RANGE_HASH(delta)];
  while (j2c_method_ranges.values[idx].end <= pc)
    idx++;
  J2cCodeFragment *ret = (j2c_method_ranges.values[idx].start <= pc
			  ? ((e_s3_services_j2c_J2cCodeFragment **)j2c_method_pointers.values)[idx]
			  : 0);
#if 0
  fprintf(stderr, "getCode(%p) => %s@%p idx = %d, distance = %d\n",
	  _pc,
	  (ret
	   ? ((char *) (ret->ovm_cname->ovm_data->values
			+ ret->ovm_cname->ovm_offset))
	   : "<null>"),
	  ret,
	  idx,
	  idx - range_hints[RANGE_HASH(delta)] );
#endif
  return ret;
}

void *j2c_topFrame(jint nc) {
  return ((J2cContext *) nc)->topFrame;
}

void *j2c_bottomFrame(jint nc) {
  return ((J2cContext *) nc)->bottomFrame;
}

// analogous to BCdead
#define RTdead { assert(!"unsupported operation"); exit(-1); }

void *getOVMContextLoc(jint nc) {
  return (void *) &((J2cContext *) nc)->ovmContext;
}

void *getInvocationArgsLoc(jint nc) {
  return (void *) &((J2cContext *) nc)->args;
}

void *getEngineSpecificPtrLoc(jint nc) {
  return getInvocationArgsLoc(nc);
}

#ifdef PRECISE_PTRSTACK
void *getPtrStackBase(jint nc) {
    return ((J2cContext*)nc)->ptrStack;
}
void *getPtrStackTop(jint nc) {
    return ((J2cContext*)nc)->gcTop;
}
void *getPtrStackLimit(jint nc) {
    return ((J2cContext*)nc)->ptrStack+PTRSTACKSIZE;
}
#else
void *getPtrStackBase(jint nc) RTdead
void *getPtrStackTop(jint nc) RTdead
void *getPtrStackLimit(jint nc) RTdead
#endif

#ifdef PRECISE_HENDERSON
jint getNextGcRecord(int _gcFrame) {
  return (jint) ((GCFrame *) _gcFrame)->next;
}

jint getGcRecord(int nc, void *callee) {
  J2cContext *ctx = (J2cContext *) nc;
  GCFrame *fr = ctx->gcTop;

  if (ctx != currentContext && callee == ctx->topFrame)
    return (jint) fr;

  while (fr && (unsigned int) fr < (unsigned int) callee)
    fr = fr->next;
  return (jint) fr;
}
  
//e_Array<Oop *> *getLocalReferences(jint gcFrame) {
e_Array *getLocalReferences(jint gcFrame) {
//  return (e_Array<Oop *> *) &((GCFrame *) gcFrame)->arrayHeader;
  return (e_Array*) &((GCFrame *) gcFrame)->arrayHeader;
}
#else
jint getNextGcRecord(jint _) RTdead
jint getGcRecord(jint _, void * __) RTdead
//e_Array<Oop *> *getLocalReferences(jint frameHandle) RTdead
e_Array *getLocalReferences(jint frameHandle) RTdead

void *getStackBase(jint nc) {
  J2cContext *ctx = (J2cContext *) nc;
  return ctx->stackBase;
}

void *getStackTop(jint nc) {
  J2cContext *ctx = (J2cContext *) nc;
  if (ctx == currentContext)
    // skip invoke_native() => getStackTop()
    return callerLocalsEnd(getCurrentActivation(2));
  else
#ifdef THREAD_UCONTEXT
    return (void *) UC_REG(ctx->uc, UC_FP);
#else
    return (void *) JB_REG(ctx->buf, JB_FP);
#endif
}

void *getSavedRegisters(jint _ctx) {
  J2cContext *ctx = (J2cContext *) _ctx;
#ifdef THREAD_UCONTEXT
  return (void *) &UC_REG(ctx->uc, 0);
#else
  return ctx->buf;
#endif
}

jint getSavedRegisterSize() {
#ifdef THREAD_UCONTEXT
  return UC_SIZE;
#else
  return sizeof(jmp_buf);
#endif
}
#endif

#if defined(PRECISE_SAFE_POINTS)
void *getAuxStackTop(int nc) {
  accurate::AuxStack *aux = &((J2cContext *) nc)->localState.auxStack;
  return aux->empty() ? getAuxStackBottom(nc) : aux->top();
}
void *getAuxStackBottom(int nc) {
  return ((J2cContext *) nc)->localState.auxStack.stack + AUXSIZE;
}
void walkStack(int nc) {
  ((J2cContext *) nc)->shouldWalkStack = 1;
}
jboolean localStateIsRunning(int nc) {
  return ((J2cContext*)nc)->localState.running;
}
jint getAuxHeaderSize() {
  return sizeof(accurate::AuxFrame);
}
// This callback serves no purpose whatsoever
void userGC(accurate::AuxStack *_) {
}
#else
void *getAuxStackTop(int nc) RTdead
void *getAuxStackBottom(int nc) RTdead
void walkStack(int nc) RTdead
jint getAuxHeaderSize() RTdead
#endif

// Should not be here!
void C(jbyte *dummy, ...) { }
