package s3.services.memory.triPizlo;

import ovm.core.execution.*;
import ovm.core.services.memory.*;
import ovm.core.domain.*;
import ovm.core.*;
import ovm.util.*;
import s3.util.*;
import s3.util.PragmaTransformCallsiteIR.BCdead;
import s3.services.bootimage.*;
import s3.core.services.memory.*;
import ovm.core.repository.JavaNames;

import java.io.*;

/** implementation of a tricolor collector.  eventually this'll be
    a production-worthy real-time GC.  for now it's just a pathetic
    hack. */
public class TheMan
    extends MemoryManager
    implements ImageAllocator.Implementation,
	       NativeConstants {
    
    static final boolean DEBUG_BUILD=true;
    static final boolean DEBUG_INIT=true;
    static final boolean DEBUG_ALLOC_SLOW=false;
    static final boolean DEBUG_ALLOC_SLOW_VERBOSE=false;
    static final boolean DEBUG_ALLOC_FAST=false;
    static final boolean DEBUG_ALLOC_FAST_VERBOSE=false;
    static final boolean DEBUG_GC=true;
    static final boolean DEBUG_WALK=false;
    static final boolean DEBUG_WALK_VERBOSE=false;
    static final boolean DEBUG_MARK=false;
    static final boolean DEBUG_MARK_VERBOSE=false;
    static final boolean DEBUG_SWEEP=false;
    static final boolean DEBUG_EXHAUSTED=true;
    static final boolean DEBUG_DUMP_HEAP=true;
    static final boolean DEBUG_PAUSE=false;
    static final boolean DEBUG_TIMER=true;
    static final boolean DEBUG_TIMER_VERBOSE=false;
    static final boolean DEBUG_PIN=false;
    static final boolean DEBUG_FREE=false;
    static final boolean DEBUG_WEIRD=true;
    static final boolean DEBUG_BARRIER=false;
    static final boolean DEBUG_INTERRUPTIBLE_STACK_SCANNING=false;
    
    static boolean debugBuild=true;
    static boolean debugInit=true;
    static boolean debugAllocSlow=true;
    static boolean debugAllocSlowVerbose=false;
    static boolean debugAllocFast=true;
    static boolean debugAllocFastVerbose=false;
    static boolean debugGC=true;
    static boolean debugWalk=true;
    static boolean debugWalkVerbose=true;
    static boolean debugMark=true;
    static boolean debugMarkVerbose=true;
    static boolean debugSweep=true;
    static boolean debugExhausted=true;
    static boolean debugDumpHeap=true;
    static boolean debugPause=true;
    static boolean debugTimer=true;
    static boolean debugTimerVerbose=true;
    static boolean debugPin=true;
    static boolean debugFree=true;
    static boolean debugWeird=true;
    static boolean debugBarrier=true;

    static final boolean PROFILE_ALLOC=false;
    static boolean profileAlloc=false;
    
    static final boolean PROFILE_BLOCK_USAGE=false;
    static final boolean PROFILE_MEM_USAGE=false;
    static boolean profileMemUsage=false;
    
    static final boolean ALLOC_STATS=false;
    static boolean allocStats=false;
    
//    static final boolean VERIFY_HEAP_INTEGRITY=false;
    static final boolean VERIFY_HEAP_INTEGRITY=true;
    
    static final boolean VERIFY_HI_PRI_ALLOC=false;
//    static final boolean VERIFY_HI_PRI_ALLOC=true;
    
    static final boolean PREDICTABLE_BARRIER=true;
//    static final boolean PREDICTABLE_BARRIER=false;
    
//    static final boolean INCREMENTAL_STACK_SCAN=false;
    static final boolean INCREMENTAL_STACK_SCAN=true;

    static boolean useImageBarrier;
    
    static void setAllDebug(boolean value) {
	debugBuild=value;
	debugInit=value;
	debugAllocSlow=value;
	debugAllocSlowVerbose=value;
	debugAllocFast=value;
	debugAllocFastVerbose=value;
	debugGC=value;
	debugWalk=value;
	debugWalkVerbose=value;
	debugMark=value;
	debugMarkVerbose=value;
	debugSweep=value;
	debugExhausted=value;
	debugDumpHeap=value;
	debugPause=value;
	debugTimer=value;
	debugTimerVerbose=value;
	debugPin=value;
	debugFree=value;
	debugWeird=value;
	debugBarrier=value;
    }
    
    static void enableSilentModeImpl() {
	setAllDebug(false);
    }
    
    public void enableSilentMode() {
	enableSilentModeImpl();
    }

    static void enableAllDebugImpl() {
	setAllDebug(true);
    }

    public void enableAllDebug() {
	enableAllDebugImpl();
    }

    static final class Nat implements NativeInterface {
/*	static native void mc_barrier_init(int imageEnd,
					   int[] continued,
					   int[] dirty); */
	
	// we use this directly when we are zeroing less than a block
	// at a time
	static native void bzero(VM_Address addr, int nb);
    }

    private static boolean inited=false;

    // design notes:
    // this is a snapshot-at-the-beginning collector that marks new
    // objects black.  I use the Technique #6 barrier that shades the
    // old referrent on a write.

    // implementation notes.  what needs to be done:
    // 1) free lists.  manage memory using that style where there are
    //    blocks and each block is dedicated to a particular size.
    // 2) I suppose I need to manage these blocks.
    // 3) write barriers.  I need a write barrier somehow.  not sure
    //    how.
    // 4) some manner of a worklist for when we're shading stuff

    // how we're going to do it:
    // 1) well, I suppose the object header is big enough that a free
    //    object has plenty of room for a 'next' pointer
    // 2) blocks can be managed with a huge bitvector
    // 3) this needs to return 'true' in needsWriteBarrier() and then
    //    it needs to implement putFieldBarrier() and aastoreBarrier().
    // 4) do it stupid way for now
    
    static final int imgPageSize = PAGE_SIZE;
    static int lastImagePageIdx;
    
    // this is just what getImageBaseAddress() returns - but we want it in a static context
    static int imageBaseAddress;

    static final int blockSize = 2048;
    
    // fraction multNum/multDen that determines the geometric sequence
    // used for allocation sizes
    static final int multNum = 2;
    static final int multDen = 1;
    
    static final int wordSize = VM_Word.widthInBytes();
    static final int alignment = 16; // the size and address of all objects must be a multiple of this number
    static final int minSize = 16;
    
    static final int maxBlockAlloc = computeMaxBlockAlloc();
    static final int sizeClassIndices = sizeClassIdx(maxBlockAlloc)+1;

    /** means that the object is untouched by GC, so cannot be assumed to
	be either available for freelisting or reachable. */
    static int WHITE = 3;
    
    /** means that the object is available for putting onto a free list */
    static final int FREE = 1;
    
    /** means that the object is marked and so is definitely reachable and
	may be on the worklist. */
    static int GREYBLACK = 2;
    
    /** means that the object is pinned.  we rely on this to be 0, so that
	an object stamped with a blueprint but untouched by GC will look to
	the GC as if it were pinned. */
    static final int PINNED = 0;
    
    static int memSize; // in bytes
    static int gcThreshold; // in blocks
    static VM_Address memory;
    
    // double indirection because multiple indices may give the same
    // size class
    static class SizeClass {
	int size;
	VM_Address objHead;
	VM_Address block;
	VM_Address blockCur;
	
	SizeClass(int size) {
	    this.size=size;
	    this.objHead=null;
	    this.block=null;
	    this.blockCur=null;
	}
    }
    
    static int[] largeBits;
    static int[] usedBits;
    static int[] sizes;
    static int[] uses; // only valid for blocks used for small objects
    static int[] blockPrev;
    static int[] blockNext;
    static int blockHead;
    
    static int nImgBits;
    static int nImgBitWords;
    static int[] continued;
    static int[] dirty;
    
    transient boolean lastBig;
//    transient PrintWriter params;

    static int nUsedBlocks; // this is essential; it is used by the GC logic.  this is also always what is reported when the memoryConsumed() method is called.
    
    static int memUsage; // only maintained if PROFILE_MEM_USAGE is set, and then it is only used for the mem usage trace.
    
    static int nBlocks;
    static int nBitWords;
    static SizeClass[] sizeClassBySize;
    static SizeClass[] sizeClasses;
    
    static int[] sizeHisto;
    static int sizeOverflow;

    static volatile boolean collecting;
    static volatile boolean marking;
    static volatile boolean stackScanning = false;
    static volatile boolean shouldPause;
    static volatile boolean threadRunning;
    static volatile boolean gcStopped=true;

    static int allocColor;
    
    static VM_Address worklist;
    static int worklistSize;
    static int worklistBits;
    static int worklistPuti;
    static int worklistGeti;
    static boolean worklistEmpty=true;

    static int marked;
    static int waited;
    
    static boolean disableConcurrency;
    static boolean stackUninterruptible;
    static boolean markUninterruptible;
    static boolean sweepUninterruptible;

    static /*final*/ Blueprint.Array intArray;
    
    private static void initLate() throws BCdead {
	try {
	    Domain ed = DomainDirectory.getExecutiveDomain();
	    Type.Context stc = ed.getSystemTypeContext();
	    Type intArrayT = stc.typeFor(JavaNames.arr_int);
	    intArray = (Blueprint.Array) ed.blueprintFor(intArrayT);
	} catch (LinkageException e) {
	    throw e.unchecked();
	}
    }

    static class Area extends VM_Area {
	public int size() { return memSize; }
	public int memoryConsumed() { return nUsedBlocks*blockSize; }
	public void addDestructor(Destructor d) {}
	public void removeDestructor(Destructor d) {}
	public Oop revive(Destructor d) { throw Executive.panic("revive() not implemented"); }
	public int destructorCount(int kind) { return 0; }
	public void walkDestructors(int kind, DestructorWalker w) {}
	public void walkDestructors(int kind, DestructorWalker w, int max) {}
    }

    static Area heap=new Area();
    
    static int inBlocks(int n) {
	return (n+blockSize-1)/blockSize;
    }
    
    static int blockRoundUp(int n) {
	return inBlocks(n)*blockSize;
    }

    static int inAlignments(int n) {
	return (n+alignment-1)/alignment;
    }
    
    static int alignmentRoundUp(int n) {
	return inAlignments(n)*alignment;
    }
    
    static int sizeClassIdx(int n) {
	return n/alignment;
    }
    
    static int iterateSize(int size) {
	size*=multNum;
	size/=multDen;
	size=(size+alignment-1)&~(alignment-1);
	return size;
    }
    
    // slow, don't use too much
    static int allocSize(int n) {
	int size=minSize;
	while (size<n) {
	    size=iterateSize(size);
	}
	return size;
    }
    
    static int computeMaxBlockAlloc() {
	int size=minSize;
	while (blockSize/iterateSize(size) >= 1) {
	    size=iterateSize(size);
	}
	return size;
    }
    
    static int offset(VM_Address ptr)
	throws PragmaNoPollcheck {
	return ptr.diff(memory).asInt();
    }
    
    static int blockIdxOff(int offset)
	throws PragmaNoPollcheck {
	return offset/blockSize;
    }
    
    static int blockIdx(VM_Address ptr)
	throws PragmaNoPollcheck {
	return blockIdxOff(offset(ptr));
    }
    
    static class Timer {
	String name;
	
	long begin;
	
	long lost;
	long lat;
	
	Timer(String name) {
	    this.name=name;
	}
	
	void ticHook(long curTime) {}
	
	final void tic() {
	    long curTime=Native.getCurrentTime();
	    if (DEBUG_TIMER_VERBOSE && debugTimerVerbose) {
		Native.print_string("triPizlo: TIMER_VERBOSE: in ");
		Native.print_string(name);
		Native.print_string(": recorded tic = ");
		Native.print_long(curTime);
		Native.print_string("\n");
	    }
	    ticHook(curTime);
	    begin=curTime;
	    lost=0;
	}
	
	void tocHook(long curTime) {}
	
	final void toc() {
	    long curTime=Native.getCurrentTime();
	    if (DEBUG_TIMER_VERBOSE && debugTimerVerbose) {
		Native.print_string("triPizlo: TIMER_VERBOSE: in ");
		Native.print_string(name);
		Native.print_string(": recorded toc = ");
		Native.print_long(curTime);
		Native.print_string("\n");
	    }
	    tocHook(curTime);
	    lat=curTime-begin;
	}
	
	void lose(long lat) {
	    lost+=lat;
	}
	
	void toc(String text) {
	    toc();
	    if (DEBUG_TIMER && debugTimer) {
		Native.print_string("triPizlo: TIMER: ");
		Native.print_string(text);
		Native.print_string(" in ");
		Native.print_long(lat);
		Native.print_string(" ns (");
		Native.print_long(lat-lost);
		Native.print_string(" ns not sleeping).\n");
	    }
	}
	
	void dump() {}
    }
    
    static class NumberLog {
	long[] buffer=new long[100000];
	int n;
	
	byte[] filename;
	
	static byte[] appendMode=new byte[]{'a',0};
	
	NumberLog(String filename) {
	    this.filename=Native.Utils.string2c_string(filename);
	}
	
	void dump() {
	    int floutInt=Native.fopen(filename,appendMode);
	    if (floutInt!=0) {
		Native.FilePtr flout=new Native.FilePtr(floutInt);
		for (int i=0;i<n;++i) {
		    Native.print_long_on(flout,buffer[i]);
		    Native.print_string_on(flout,"\n");
		}
		Native.fclose(flout);
	    }
	    n=0;
	}

	void log(long value) {
	    buffer[n++]=value;
	    if (n==buffer.length) {
		dump();
	    }
	}
    }
    
    static class LoggingTimer extends Timer {
	NumberLog ticLog;
	NumberLog tocLog;
	
	LoggingTimer(String name,String filename) {
	    super(name);
	    ticLog=new NumberLog("tic_"+filename);
	    tocLog=new NumberLog("toc_"+filename);
	}
	
	void ticHook(long curTime) {
	    ticLog.log(curTime);
	}
	
	void tocHook(long curTime) {
	    tocLog.log(curTime);
	}
	
	void dump() {
	    ticLog.dump();
	    tocLog.dump();
	}
    }
    
    static long totalGCTime=0;
    static Timer outer=new Timer("outer");
    static Timer inner=new Timer("inner");
    static Timer pollLat=new Timer("pollLat");
    static Timer stackScan=new Timer("stackScan");
    static Timer debug=new Timer("debug");
    
    static NumberLog allocTimestamp;
    static NumberLog allocSize;
    
    static NumberLog memUsageTimestamp;
    static NumberLog blockUsageValue;
    static NumberLog memUsageValue;
    
    static GCBits.Model gcBits;
    static FreeList freeList;
    
    static boolean noBarrier;
    
    public TheMan(String memSize,
		  String gcThreshold,
		  String noBarrier) {
	this(CommandLine.parseSize(memSize),
	     CommandLine.parseSize(gcThreshold),
	     noBarrier!=null);
    }
    
    public TheMan(int memSize,
		  int gcThreshold,
		  boolean noBarrier) {
	if (inited) {
	    throw new Error("Already have a triPizlo.TheMan");
	}
	inited=true;
	
	ImageAllocator.override(this);
	
	gcBits=(GCBits.Model)ObjectModel.getObjectModel();
	freeList=(FreeList)ObjectModel.getObjectModel();

	TheMan.noBarrier=noBarrier;
	TheMan.memSize=blockRoundUp(memSize);
	TheMan.gcThreshold=gcThreshold/blockSize;
	nBlocks=memSize/blockSize;
	nBitWords=((nBlocks+31)/32);
	
	imageBaseAddress = getImageBaseAddress();
	
	if (DEBUG_BUILD && debugBuild) {
	    System.out.println("triPizlo: BUILD: creating TheMan.");
	    System.out.println("triPizlo: BUILD: noBarrier = "+noBarrier);
	    if (noBarrier) {
		System.out.println("triPizlo: BUILD: !!! WARNING !!! BARRIER TURNED OFF !!! WARNING !!!");
		System.out.println("triPizlo: WARNING !!! REQUEST TO TURN OFF THE BARRIER IGNORED !!!");
	    }
	    System.out.println("triPizlo: BUILD: imgPageSize = "+imgPageSize);
	    System.out.println("triPizlo: BUILD: blockSize = "+blockSize);
	    System.out.println("triPizlo: BUILD: multNum = "+multNum);
	    System.out.println("triPizlo: BUILD: multDen = "+multDen);
	    System.out.println("triPizlo: BUILD: wordSize = "+wordSize);
	    System.out.println("triPizlo: BUILD: alignment = "+alignment);
	    System.out.println("triPizlo: BUILD: minSize = "+minSize);
	    System.out.println("triPizlo: BUILD: maxBlockAlloc = "+maxBlockAlloc);
	    System.out.println("triPizlo: BUILD: sizeClassIndices = "+sizeClassIndices);
	    System.out.println("triPizlo: BUILD: memSize = "+memSize);
	    System.out.println("triPizlo: BUILD: gcThreshold = "+this.gcThreshold);
	    System.out.println("triPizlo: BUILD: nBlocks = "+nBlocks);
	    System.out.println("triPizlo: BUILD: nBitWords = "+nBitWords);
	    System.out.println("triPizlo: BUILD: imageBaseAddress = "+getImageBaseAddress());
	}
	
	sizeClassBySize=new SizeClass[sizeClassIndices];
	sizeClasses=new SizeClass[0];
	for (int i=0;i<sizeClassIndices;++i) {
	    int n=i*alignment;
	    int size=allocSize(n);
	    if (size>blockSize) {
		throw new Error();
	    }
	    if (i>0 && sizeClassBySize[i-1].size==size) {
		sizeClassBySize[i]=sizeClassBySize[i-1];
	    } else {
		if (DEBUG_BUILD && debugBuild) {
		    System.out.println("triPizlo: BUILD: size class for size range "+n+".."+size);
		}
		
		SizeClass fl=new SizeClass(size);
		
		sizeClassBySize[i]=fl;

		SizeClass[] newSizeClasses=new SizeClass[sizeClasses.length+1];
		System.arraycopy(sizeClasses,0,
				 newSizeClasses,0,
				 sizeClasses.length);
		newSizeClasses[sizeClasses.length]=fl;
		sizeClasses=newSizeClasses;
	    }
	}
	
	largeBits=new int[nBitWords+1];
	usedBits=new int[nBitWords+1];
	sizes=new int[nBlocks];
	uses=new int[nBlocks];
	blockNext=new int[nBlocks];
	blockPrev=new int[nBlocks];
	blockHead=0;

	for (int i=0;i<nBlocks;++i) {
	    blockPrev[i]=i-1;
	    if (i==nBlocks-1) {
		blockNext[i]=-1;
	    } else {
		blockNext[i]=i+1;
	    }
	}

	for (worklistSize=1;worklistSize<memSize/minSize;worklistSize*=2) ;
	worklistBits=worklistSize-1;
	if (DEBUG_BUILD && debugBuild) {
	    Native.print_string("triPizlo: BUILD: worklistSize = ");
	    Native.print_int(worklistSize);
	    Native.print_string("\n");
	    
	    Native.print_string("triPizlo: BUILD: worklistBits = ");
	    Native.print_int(worklistBits);
	    Native.print_string("\n");
	}
	
	allocColor=WHITE;

/*	try {
	    params = new PrintWriter(new FileOutputStream
				     ("mc_barrier_params.h"),
				     true);
	    params.println("#define OVM_PAGE_SIZE " + imgPageSize);
	    params.println("#define HEAP_START 0x" +
			   Integer.toHexString(getImageBaseAddress()));
	    params.println("#define IMAGE_CONTINUED_RANGES \\");

	    Driver.gen_ovm_c.println("#include <mc_barrier_params.h>");
	    Driver.gen_ovm_c.println("#include <mc_barrier.h>");
	    Driver.gen_ovm_c.flush();
	} catch (IOException e) {
	    throw new RuntimeException(e);
	} */
	// FIXME: WHY does timedWait run fast?
	// Is there something wrong with our regression tests, or
	// something wrong with the RTGC?
	RuntimeExports.defineVMProperty("org.ovmj.timedWaitRunsFast", true);
    }

    static int[] allocIntArrayRaw(int size) {
	int totalSize=(int)intArray.computeSizeFor(size);
	VM_Address result=Native.getmem(totalSize);
	Nat.bzero(result,totalSize);
	intArray.stamp(result,size);
	return (int[])result.asAnyObject();
    }
    
    static boolean inHeap(VM_Address addr) {
	return addr.diff(memory).uLT(VM_Word.fromInt(memSize));
    }
    
    static boolean inHeap(Oop oop) {
	return inHeap(VM_Address.fromObject(oop));
    }
    
    static void verifyPointer(VM_Address addr) {
	if (inHeap(addr)) {
	    int blockIdx=blockIdx(addr);
	    VM_Address blockBase=memory.add(blockIdx*blockSize);
	    int size=sizes[blockIdx];
	    int offset=addr.diff(blockBase).asInt();
	    if (Bits.getBit(largeBits,blockIdx)) {
		if (size<=0) {
		    Native.print_string("VERIFY_HEAP_INTEGRITY: ERROR: saw offset pointer ");
		    Native.print_ptr(addr);
		    Native.print_string("!  this is probably a large object pointer, but it points at a continued block.\n");
		}
		if (offset!=0) {
		    Native.print_string("VERIFY_HEAP_INTEGRITY: ERROR: saw offset pointer ");
		    Native.print_ptr(addr);
		    Native.print_string("!  this is probably a large object pointer, but it does not point at the beginning of a block.\n");
		}
	    } else if ((offset%size)!=0) {
		Native.print_string("VERIFY_HEAP_INTEGRITY: ERROR: saw offset pointer ");
		Native.print_ptr(addr);
		Native.print_string("!  should be at ");
		Native.print_ptr(blockBase.add((offset/size)*size));
		Native.print_string(", with the block base at ");
		Native.print_ptr(blockBase);
		Native.print_string(" and an allocation size of ");
		Native.print_int(size);
		Native.print_string("\n");
	    }
	    if (gcBits.getGCBits(addr)==FREE) {
		Native.print_string("VERIFY_HEAP_INTEGRITY: ERROR: saw pointer to a free object at ");
		Native.print_ptr(addr);
		Native.print_string("\n");
	    }
	    if (!Bits.getBit(usedBits,blockIdx)) {
		Native.print_string("VERIFY_HEAP_INTEGRITY: ERROR: saw pointer into unused block at ");
		Native.print_ptr(addr);
		Native.print_string("\n");
	    }
	} else {
	    Native.print_string("VERIFY_HEAP_INTEGRITY: ERROR: pointer ");
	    Native.print_ptr(addr);
	    Native.print_string(" is not in the heap!\n");
	}
    }
    
    static void verifyPointer(Oop oop) {
	verifyPointer(VM_Address.fromObject(oop));
    }
    
    public final void boot(boolean useImageBarrier) {
	this.useImageBarrier = useImageBarrier;
	
	if (DEBUG_INIT && debugInit) {
	    Native.print_string("triPizlo: INIT: booting.\n");
	    if (noBarrier) {
		Native.print_string("triPizlo: INIT: !!! WARNING !!! BARRIER TURNED OFF !!! WARNING !!!\n");
		Native.print_string("triPizlo: WARNING !!! REQUEST TO TURN OFF THE BARRIER IGNORED !!!\n");		
	    }
	}
	memory=Native.getmem(memSize+blockSize);
	if (DEBUG_INIT && debugInit) {
	    Native.print_string("triPizlo: INIT: heap allocated at ");
	    Native.print_ptr(memory);
	    Native.print_string(".\n");
	}
	memory=memory.asWord().add(blockSize-1).div(blockSize).mul(blockSize).asAddress();
	if (DEBUG_INIT && debugInit) {
	    Native.print_string("triPizlo: INIT: usable heap base at ");
	    Native.print_ptr(memory);
	    Native.print_string(".\n");
	}
	Nat.bzero(memory,memSize);
	    
	worklist=Native.getmem(worklistSize*wordSize);
//	if (useImageBarrier) {
	    nImgBits=
	        (Native.getImageEndAddress().asInt()-getImageBaseAddress())
	        /imgPageSize;
	    nImgBitWords=(nImgBits+31)/32;
	    

	    
	    if (DEBUG_INIT && debugInit) {
		Native.print_string("triPizlo: INIT: imageBaseAddress = ");
		Native.print_ptr(Native.getImageBaseAddress());
		Native.print_string("\n");
	    
		Native.print_string("triPizlo: INIT: imageEndAddress = ");
		Native.print_ptr(Native.getImageEndAddress());
		Native.print_string("\n");
	    
		Native.print_string("triPizlo: INIT: nImgBits = ");
		Native.print_int(nImgBits);
		Native.print_string("\n");
	    
		Native.print_string("triPizlo: INIT: nImgBitWords = ");
		Native.print_int(nImgBitWords);
		Native.print_string("\n");
		

				
	    }
//	    continued=allocIntArrayRaw(nImgBitWords+1);

            lastImagePageIdx = -1; // this is set to disable the image SW barrier for the assignment
                                   // of the dirty vector ; it works, because the dirty vector is allocated
                                   // out of garbage collected heap
	    dirty=allocIntArrayRaw(nImgBitWords+1);
	    lastImagePageIdx = imageAddressToPageIndex(Native.getImageEndAddress());
	    
	    if (DEBUG_INIT && debugInit) {
                Native.print_string("triPizlo: INIT: lastImagePageIdx = ");
		Native.print_int(lastImagePageIdx);
		Native.print_string("\n");	    
	    
		Native.print_string("triPizlo: INIT: image meta-data initialized.\n");
	    }
/*	    Nat.mc_barrier_init(Native.getImageEndAddress().asInt(),
				continued, dirty);
	    if (DEBUG_INIT && debugInit) {
		Native.print_string("triPizlo: INIT: image barrier initialized.\n");
	    }
	    Native.mprotect(VM_Address.fromInt(getImageBaseAddress()),
			    nImgBits*imgPageSize,
			    PROT_READ);
	    if (DEBUG_INIT && debugInit) {
		Native.print_string("triPizlo: INIT: image write-protected.\n");
		Native.print_string("triPizlo: INIT: ready for heap allocation.\n"); 
	    } */
//	}

        if (DEBUG_INTERRUPTIBLE_STACK_SCANNING) {
	    Native.print_string("triPizlo: INIT: INTERRUPTIBLE_STACK_SCANNING: stackUninterruptible =  ");
	    Native.print_string( stackUninterruptible ? "true" : "false");
	    Native.print_string("\n");        
	    
	    Native.print_string("triPizlo: INIT: INTERRUPTIBLE_STACK_SCANNING: INCREMENTAL_STACK_SCAN =  ");
	    Native.print_string( INCREMENTAL_STACK_SCAN ? "true" : "false");
	    Native.print_string("\n");        
        }    
    }
    
    static String colorName(int color) {
	if (color==WHITE) {
	    return "W";
	} else if (color==GREYBLACK) {
	    return "G";
	} else if (color==FREE) {
	    return "F";
	} else if (color==PINNED) {
	    return "P";
	} else {
	    return "?";
	}
    }
    
    static void dumpHeap() {
	if (DEBUG_DUMP_HEAP && debugDumpHeap) {
	    for (int bi=0;bi<nBlocks;) {
		int increment=1;
		if (Bits.getBit(usedBits,bi)) {
		    Native.print_string("triPizlo: DUMP_HEAP: #");
		    Native.print_int(bi);
		    Native.print_string(" base=");
		    Native.print_ptr(memory.add(bi*blockSize));
		    Native.print_string(" size=");
		    Native.print_int(sizes[bi]);
		    Native.print_string(" next=");
		    Native.print_int(blockNext[bi]);
		    Native.print_string(" ");
		    if (Bits.getBit(largeBits,bi)) {
			Native.print_string("L");
		    } else {
			Native.print_string("S");
		    }
		    if (Bits.getBit(largeBits,bi)) {
			Native.print_string(" color=");
			Native.print_string(colorName(gcBits.getGCBits(memory.add(bi*blockSize))));
			increment=sizes[bi];
		    } else {
			Native.print_string(" colors=");
			for (int oi=0;oi<blockSize/sizes[bi];++oi) {
			    Native.print_string(colorName(gcBits.getGCBits(memory.add(bi*blockSize).add(oi*sizes[bi]))));
			}
		    }
		    Native.print_string("\n");
		}
		bi+=increment;
	    }
	    for (int sci=0;sci<sizeClasses.length;++sci) {
		// FIXME: this is an incomplete dump
		SizeClass sc=sizeClasses[sci];
		Native.print_string("triPizlo: DUMP_HEAP: size class for ");
		Native.print_int(sc.size);
		Native.print_string(": objects:");
		for (VM_Address cur=sc.objHead;cur!=null;cur=freeList.getNext(cur)) {
		    Native.print_string(" ");
		    Native.print_ptr(cur);
		}
		Native.print_string("\n");
	    }
	}
    }
    
    static boolean alreadyExhausted=false;
    static Error memoryExhausted(int size) {
	// the logic for alreadyExhausted is funky.  the key to understanding why
	// the code works the way it does is to notice the call to getMemImpl,
	// which does not return, but instead reenters memoryExhausted().  this
	// is what the alreadyExhausted flag is for.
	if (!alreadyExhausted) {
	    alreadyExhausted=true;
	    if (DEBUG_DUMP_HEAP && debugDumpHeap) {
		dumpHeap();
	    }
	    if (DEBUG_EXHAUSTED && debugExhausted) {
		Native.print_string("triPizlo: EXHAUSTED: exhausted memory with request for ");
		Native.print_int(size);
		Native.print_string(" bytes.  Re-executing memory request with all debug flags enabled.\n");
		enableAllDebugImpl();
		// this works because if memoryExhausted() is called, the memory allocation
		// request did not lead to anything being changed, other than GC being notified.
		// but since we are in atomic code, that makes no difference.  so, re-executing
		// the allocation request will lead to the same results, modulo the fact that
		// alreadyExhausted will be set to true, so this won't execute twice.
		getMemImpl(size);
		
		// should never get here, because getMemImpl() should call
		// memoryExhausted() again, which should either panic or throw.
		throw Executive.panic("in memoryExhausted: "+
				      "getMemImpl returned successfully on second "+
				      "try, which shouldn't happen.");
	    }
	}
	if (alreadyExhausted) {
	    alreadyExhausted=false;
	}
	// FIXME: I may want to change this to use outOfMemory()
	throw Executive.panic("Out Of Memory");
    }

    public VM_Address getMem(int size) {
	throw Executive.panic("triPizlo does not implement getMem directly.");
    }

    static VM_Address getMemImpl(int size) {
	if (DEBUG_ALLOC_FAST && debugAllocFast) {
	    Native.print_string("triPizlo: ALLOC_FAST: allocating object of size ");
	    Native.print_int(size);
	    Native.print_string("\n");
	}
	
	if (PROFILE_ALLOC && profileAlloc) {
	    profileAlloc=false;
	    allocTimestamp.log(Native.getCurrentTime());
	    allocSize.log(size);
	    profileAlloc=true;
	}
	
	if (VERIFY_HI_PRI_ALLOC && !gcStopped) {
	    gcStopped=true;
	    new Error().printStackTrace();
	    throw Executive.panic("attempt to allocate memory when the GC is not stopped");
	}
	
	if (ALLOC_STATS && allocStats) {
	    if (size<sizeHisto.length) {
		sizeHisto[size]++;
	    } else {
		sizeOverflow++;
	    }
	}

	if (size<=maxBlockAlloc) {
	    return getMemSmall(size);
	} else {
	    return getMemLarge(size);
	}
    }
    
    public Oop allocate(Blueprint.Scalar bp) throws PragmaAtomic {
	return stampGCBits(bp.stamp(getMemImpl(bp.getFixedSize())));
    }
    
    public Oop allocateArray(Blueprint.Array bp, int len) throws PragmaAtomic {
	long size = bp.computeSizeFor(len);
	if (size <= memSize)
	    return stampGCBits(bp.stamp(getMemImpl((int) size), len));
	else
	    throw outOfMemory();
    }
    
    public Oop clone(Oop oop) throws PragmaAtomic {
        Blueprint bp = oop.getBlueprint();
        return stampGCBits(bp.clone(oop, getMemImpl(bp.getVariableSize(oop))));
    }

    Oop stampGCBits(Oop oop) {
	if (VERIFY_HEAP_INTEGRITY) {
	    verifyPointer(oop);
	}
	GCBits goop=(GCBits)oop.asAnyOop();
	goop.setGCBits(allocColor);
	return goop;
    }
    
    static VM_Address getMemSmall(int size) {
	int alignments=inAlignments(size);
	SizeClass sc=sizeClassBySize[alignments];
	int realSize=sc.size;
	VM_Address result=sc.objHead;
	
	// once I make it so that instead of having one linked list for all
	// objects that are free of a particular size, to having a linked
	// list of blocks that have free lists, then we could rewrite this
	// to unify the handling of pointer-bump blocks and freelist blocks.
	// basically, if 'null' next pointer could mean bump the pointer.
	// this means that you have to have a check when you read off the
	// next pointer, but it also means that you DON'T have to have any
	// checks to see if the block is suitable for freelist allocation
	// or not.  such a fast path may or may not look faster, but the
	// total code will be less, leading to more inlining opportunities.

	if (result!=null) {
	    if (DEBUG_ALLOC_FAST_VERBOSE && debugAllocFastVerbose) {
		Native.print_string("triPizlo: ALLOC_FAST_VERBOSE: found freelist, proceeding.\n");
	    }
	    
	    if (DEBUG_ALLOC_FAST_VERBOSE && debugAllocFastVerbose) {
		Native.print_string("triPizlo: ALLOC_FAST_VERBOSE: updating freelist.\n");
	    }

	    sc.objHead=freeList.getNext(result);
	    if (sc.objHead!=null) {
		freeList.setPrev(sc.objHead,null);
	    }

	    // zero out freelist link pointers (this may not be necessary)
	    freeList.setPrev(result,null);
	    freeList.setNext(result,null);
	} else {
	    if (sc.block==null) {
		if (DEBUG_ALLOC_SLOW && debugAllocSlow) {
		    Native.print_string("triPizlo: ALLOC_SLOW: need to find new block.\n");
		}

		if (blockHead==-1) {
		    throw memoryExhausted(size);
		}
		int curBlock=blockHead;

		sc.block=memory.add(curBlock*blockSize);
		sc.blockCur=sc.block;
		
		if (DEBUG_ALLOC_SLOW && debugAllocSlow) {
		    Native.print_string("triPizlo: ALLOC_SLOW: got a block: ");
		    Native.print_int(curBlock);
		    Native.print_string(", with address: ");
		    Native.print_ptr(sc.block);
		    Native.print_string("\n");
		}

		blockHead=blockNext[curBlock];

		// this may not be necessary
		blockNext[curBlock]=-1;
		blockPrev[curBlock]=-1;
		
		if (blockHead>=0) {
		    blockPrev[blockHead]=-1;
		}
		
		nUsedBlocks++;
		Bits.setBit(usedBits,curBlock);
		sizes[curBlock]=realSize;
		
		if (PROFILE_BLOCK_USAGE && !PROFILE_MEM_USAGE) {
		    observeMemUsageChanged(); // danger: this may allocate...  but I think that'll be fine.
		}
	    }
	    
	    if (DEBUG_ALLOC_FAST_VERBOSE && debugAllocFastVerbose) {
		Native.print_string("triPizlo: ALLOC_FAST_VERBOSE: allocating by bumping pointer from block: ");
		Native.print_ptr(sc.block);
		Native.print_string("\n");
	    }

	    result=sc.blockCur;
	    sc.blockCur=result.add(realSize);
	    
	    if ((blockSize-sc.blockCur.diff(sc.block).asInt())
		< realSize) {
		if (DEBUG_ALLOC_SLOW && debugAllocSlow) {
		    Native.print_string("triPizlo: ALLOC_SLOW: block is exhausted.\n");
		}
		sc.blockCur=null;
		sc.block=null;
	    }
	}

	if (DEBUG_ALLOC_FAST && debugAllocFast) {
	    Native.print_string("triPizlo: ALLOC_FAST: object allocated: ");
	    Native.print_ptr(result);
	    Native.print_string("\n");
	}
	
	if (PROFILE_MEM_USAGE) {
	    memUsage+=size;
	    observeMemUsageChanged();
	}
	
	uses[blockIdx(result)]++;
	
	return result;
    }
    
    static VM_Address getMemLarge(int size) {
	if (DEBUG_ALLOC_SLOW && debugAllocSlow) {
	    Native.print_string("triPizlo: ALLOC_SLOW: in large object slow path for size ");
	    Native.print_int(size);
	    Native.print_string("\n");
	}
	
	int sizeInBlocks=inBlocks(size);
	if (DEBUG_ALLOC_SLOW && debugAllocSlow) {
	    Native.print_string("triPizlo: ALLOC_SLOW: need ");
	    Native.print_int(sizeInBlocks);
	    Native.print_string(" consecutive blocks.\n");
	}

	int cur=0;
	for (;;) {
	    if (DEBUG_ALLOC_SLOW && debugAllocSlow) {
		Native.print_string("triPizlo: ALLOC_SLOW: looking for free block.\n");
	    }
	    
	    cur=Bits.findClr(usedBits,cur);
	    if (cur<0 || cur>=nBlocks) {
		break;
	    }
	    
	    if (DEBUG_ALLOC_SLOW && debugAllocSlow) {
		Native.print_string("triPizlo: ALLOC_SLOW: found potential start: ");
		Native.print_int(cur);
		Native.print_string("\n");
	    }
	    
	    // found a span of free blocks.  are there enough free blocks
	    // to satisfy the request?
	    
	    int end=Bits.findSet(usedBits,nBitWords,cur+1);
	    if (DEBUG_ALLOC_SLOW_VERBOSE && debugAllocSlowVerbose) {
		Native.print_string("triPizlo: ALLOC_SLOW_VERBOSE: findSet returned ");
		Native.print_int(end);
		Native.print_string(", while nBlocks = ");
		Native.print_int(nBlocks);
		Native.print_string("\n");
	    }
	    if (end<0 || end>nBlocks) {
		end=nBlocks;
	    }
	    
	    if (DEBUG_ALLOC_SLOW && debugAllocSlow) {
		Native.print_string("triPizlo: ALLOC_SLOW: end of span: ");
		Native.print_int(end);
		Native.print_string("\n");
	    }
	    
	    if (end-cur >= sizeInBlocks) {
		// return this
		VM_Address result=memory.add(cur*blockSize);
		
		if (DEBUG_ALLOC_SLOW && debugAllocSlow) {
		    Native.print_string("triPizlo: ALLOC_SLOW: all good, returning: ");
		    Native.print_ptr(result);
		    Native.print_string("\n");
		}
		
		// first, mark it as used.
		// WARNING: this is atomic and slow!  yikes!
		for (int i=cur;i<cur+sizeInBlocks;++i) {
		    Bits.setBit(usedBits,i);
		    Bits.setBit(largeBits,i);
		    if (blockPrev[i]<0) {
			blockHead=blockNext[i];
		    } else {
			blockNext[blockPrev[i]]=blockNext[i];
		    }
		    if (blockNext[i]>=0) {
			blockPrev[blockNext[i]]=blockPrev[i];
		    }
		    blockNext[i]=-1;
		    blockPrev[i]=-1;
		    sizes[i]=-1; // this marks the block as a continuation
		}
		
		// next, set the size
		sizes[cur]=sizeInBlocks;
		
		// do some book keeping
		nUsedBlocks+=sizeInBlocks;
		
		if (PROFILE_MEM_USAGE) {
		    memUsage+=size;
		}
		
		observeMemUsageChanged();
		
		// return the mo fo
		return result;
	    }
	    
	    // failed, keep looking.
	    cur=end;
	    if (cur>=nBlocks) {
		break;
	    }
	}
	
	if (DEBUG_ALLOC_SLOW && debugAllocSlow) {
	    Native.print_string("triPizlo: ALLOC_SLOW: could not find sufficiently large span.\n");
	}
	
	throw memoryExhausted(size);
    }
    
    static Object pollMonitor=new Object();
    
    static void pollSlow() {
	try {
	    boolean oldGCStopped=gcStopped;
	    gcStopped=true; /* needs to be before tic(), see below */
	    pollLat.tic();
	    synchronized (pollMonitor) {
		while (shouldPause) {
		    waited++;
		    pollMonitor.wait();
		}
	    }
	    pollLat.toc();
	    gcStopped=oldGCStopped; /* this needs to come after the toc(), because
				       pollLat may be a LoggingTimer, and a LoggingTimer
				       may want to allocate memory. */
	    outer.lose(pollLat.lat);
	    inner.lose(pollLat.lat);
	    stackScan.lose(pollLat.lat);
	} catch (InterruptedException e) {
	    throw Executive.panic("I got an InterruptedException in pollSlow().");
	}
    }

    static void pollStackScanning() {
	if (!disableConcurrency && !stackUninterruptible && shouldPause) {

            if (DEBUG_INTERRUPTIBLE_STACK_SCANNING) {
    	      Native.print_string("triPizlo: pollStackScanning(): calling pollSlow()...\n");
    	      Native.print_string("DEBUG_BARRIER is ");
    	      Native.print_string( DEBUG_BARRIER ? "true" : "false");
    	      Native.print_string("\n");
    	      Native.print_string("debugBarrier is ");
    	      Native.print_string( debugBarrier ? "true" : "false");
    	      Native.print_string("\n");
            }
	
	    pollSlow();
	}
    }
    
    static void pollMarking() {
	if (!disableConcurrency && !markUninterruptible && shouldPause) {
	    pollSlow();
	}
    }
    
    static void pollSweeping() {
	if (!disableConcurrency && !sweepUninterruptible && shouldPause) {
	    pollSlow();
	}
    }
    
    static void walkObject(Oop oop,
			   Blueprint bp) {
	if (DEBUG_WALK && debugWalk) {
	    Native.print_string("triPizlo: WALK: Walking object ");
	    Native.print_ptr(VM_Address.fromObject(oop));
	    Native.print_string(", which has blueprint at ");
	    Native.print_ptr(VM_Address.fromObject(bp));
	    Native.print_string(", and the debug string is ");
	    byte[] str=bp.get_dbg_string();
	    Native.print_bytearr_len(str,str.length);
	    Native.print_string("\n");
	}
	for (int i = 0;
	     i < ObjectModel.getObjectModel().maxReferences();
	     i++) {
	    if (DEBUG_WALK_VERBOSE && debugWalkVerbose) {
		Native.print_string("triPizlo: WALK_VERBOSE: walking object model reference #");
		Native.print_int(i);
		Native.print_string("\n");
	    }
	    mark(VM_Address.fromObject(oop.getReference(i)));
	}
	if (bp.isArray()) {
	    if (DEBUG_WALK_VERBOSE && debugWalkVerbose) {
		Native.print_string("triPizlo: WALK_VERBOSE: object is an array.\n");
	    }
	    Blueprint.Array abp = bp.asArray();
	    if (abp.getComponentBlueprint().isReference()) {
		VM_Address p = abp.addressOfElement(oop, 0);
		VM_Address pend = p.add(VM_Word.widthInBytes()
					* abp.getLength(oop));
		if (DEBUG_WALK_VERBOSE && debugWalkVerbose) {
		    Native.print_string("triPizlo: WALK_VERBOSE: object is an array of pointers, with begin = ");
		    Native.print_ptr(p);
		    Native.print_string(" and end = ");
		    Native.print_ptr(pend);
		    Native.print_string("\n");
		}
		while (p.uLT(pend)) {
		    pollMarking();
		    if (DEBUG_WALK_VERBOSE && debugWalkVerbose) {
			Native.print_string("triPizlo: WALK_VERBOSE: walking array, at ");
			Native.print_ptr(p);
			Native.print_string(", with end = ");
			Native.print_ptr(pend);
			Native.print_string("\n");
		    }
		    mark(p.getAddress());
		    p=p.add(VM_Word.widthInBytes());
		}
	    }
	}
	else {
	    VM_Address base = VM_Address.fromObject(oop);
	    int[] offset = bp.getRefMap();
	    if (DEBUG_WALK_VERBOSE && debugWalkVerbose) {
		Native.print_string("triPizlo: WALK_VERBOSE: walking object with bp = ");
		Native.print_ptr(VM_Address.fromObject(bp));
		Native.print_string(", ref map at ");
		Native.print_ptr(VM_Address.fromObject(offset));
		Native.print_string(", and ");
		Native.print_int(offset.length);
		Native.print_string(" pointers.\n");
	    }
	    for (int i = 0; i < offset.length; i++) {
		if (DEBUG_WALK_VERBOSE && debugWalkVerbose) {
		    Native.print_string("triPizlo: WALK_VERBOSE: walking pointer #");
		    Native.print_int(i);
		    Native.print_string(", which is at offset ");
		    Native.print_int(offset[i]);
		    Native.print_string("\n");
		}
		VM_Address ptr=base.add(offset[i]);
		if (DEBUG_WALK_VERBOSE && debugWalkVerbose) {
		    Native.print_string("triPizlo: WALK_VERBOSE: the address of the pointer is ");
		    Native.print_ptr(ptr);
		    Native.print_string("\n");
		}
		mark(ptr.getAddress());
	    }
	}
	if (DEBUG_WALK_VERBOSE && debugWalkVerbose) {
	    Native.print_string("triPizlo: WALK_VERBOSE: walked object.\n");
	}
    }
    
    static void walkObject(VM_Address addr) {
	Oop oop = addr.asOop();
	Blueprint bp = oop.getBlueprint();
	walkObject(oop,bp);
    }
    
    static int objectSize(VM_Address addr) {
	Oop oop=addr.asOop();
	Blueprint bp=oop.getBlueprint();
	if (bp==null) {
	    return 0;
	} else {
	    return bp.getVariableSize(oop);
	}
    }

    static int imageAddressToPageIndex( VM_Address addr ) {
      return (addr.asInt() - imageBaseAddress ) / imgPageSize;
//      return addr.diff(getImageBaseAddress().asAddress()).div(imgPageSize).asInt();	    
    }
    
    static VM_Address pageIndexToImageAddress( int pageIdx ) {
      return VM_Address.fromInt(imageBaseAddress + pageIdx*imgPageSize);
    }
    

    static ExtentWalker mc = new ExtentWalker.Scanner() {
	    protected void walk (Oop oop, Blueprint bp) {
		pollMarking();
		walkObject(oop, bp);
	    }
	    
	    /*
	      start walking image from the beginning of the page of index
	      pageIdx, walk all dirty objects starting at this page ; return
	      the index of the page where next scanning should be done
	      (it is the first following non-continued page)
            */
	      

	};

    protected int walkImageFromPage(int pageIdx) {

      return walkImageFromAddress( pageIndexToImageAddress(pageIdx) );
    }
	    
    protected int walkImageFromAddress(VM_Address addr) {
            
      return imageAddressToPageIndex( mc.walkRet( addr, addr.add(imgPageSize) ) );	    
    }    



    public boolean caresAboutDisableConcurrency() { return true; }
    public void setDisableConcurrency(boolean disableConcurrency) {
	TheMan.disableConcurrency=disableConcurrency;
    }
    
    public boolean caresAboutStackUninterruptible() { return true; }
    public void setStackUninterruptible(boolean stackUninterruptible) {
    
        if (DEBUG_INTERRUPTIBLE_STACK_SCANNING) {
	    Native.print_string("triPizlo: INTERRUPTIBLE_STACK_SCANNING: setStackUninterruptible( ");
	    Native.print_string( stackUninterruptible ? "true" : "false");
	    Native.print_string(")\n");        
        }    
	TheMan.stackUninterruptible=stackUninterruptible;
    }
    
    public boolean caresAboutMarkUninterruptible() { return true; }
    public void setMarkUninterruptible(boolean markUninterruptible) {
    
	TheMan.markUninterruptible=markUninterruptible;
    }
    
    public boolean caresAboutSweepUninterruptible() { return true; }
    public void setSweepUninterruptible(boolean sweepUninterruptible) {
	TheMan.sweepUninterruptible=sweepUninterruptible;
    }
    
    public boolean caresAboutEnableTimeTrace() { return true; }
    public void setEnableTimeTrace(boolean enableTimeTrace) throws PragmaAtomic {
	if (enableTimeTrace) {
	    pollLat=new LoggingTimer("pollLat","gc_pause.txt");
	    stackScan=new LoggingTimer("stackScan","stack_scan.txt");
	} else {
	    pollLat=new Timer("pollLat");
	    stackScan=new Timer("stackScan");
	}
    }
    
    public void setGCThreshold(long gcThreshold) throws PragmaAtomic {
	this.gcThreshold=(int)gcThreshold/blockSize;
    }
    
    public void enableAllocProfiling() throws PragmaAtomic {
	if (PROFILE_ALLOC) {
	    if (profileAlloc) {
		return;
	    }
	    allocTimestamp=new NumberLog("alloc_timestamp.txt");
	    allocSize=new NumberLog("alloc_size.txt");
	    profileAlloc=true;
	}
    }
    
    public void enableSizeHisto(int sizeHistoSize) throws PragmaAtomic {
	sizeHisto=allocIntArrayRaw(sizeHistoSize);
	sizeOverflow=0;
	allocStats=true;
    }
    
    public void enableProfileMemUsage() throws PragmaAtomic {
	if (PROFILE_BLOCK_USAGE || PROFILE_MEM_USAGE) {
	    if (profileMemUsage) {
		return;
	    }
	    memUsageTimestamp=new NumberLog("mem_usage_timestamp.txt");
	    blockUsageValue=new NumberLog("block_usage_value.txt");
	    memUsageValue=new NumberLog("mem_usage_value.txt");
	    profileMemUsage=true;
	}
    }
    
    static void observeMemUsageChanged() {
	if ((PROFILE_BLOCK_USAGE || PROFILE_MEM_USAGE)
	    && profileMemUsage) {
	    boolean lastGCStopped=gcStopped;
	    gcStopped=true;
	    profileMemUsage=false;
	    memUsageTimestamp.log(Native.getCurrentTime());
	    if (PROFILE_BLOCK_USAGE) {
		blockUsageValue.log(nUsedBlocks);
	    }
	    if (PROFILE_MEM_USAGE) {
		memUsageValue.log(memUsage);
	    }
	    gcStopped=lastGCStopped;
	    profileMemUsage=true;
	}
    }
    
    public void vmShuttingDown() {
	pollLat.dump();
	stackScan.dump();
	if (PROFILE_ALLOC && profileAlloc) {
	    profileAlloc=false;
	    allocTimestamp.dump();
	    allocSize.dump();
	}
	if ((PROFILE_MEM_USAGE || PROFILE_BLOCK_USAGE)
	    && profileMemUsage) {
	    profileMemUsage=false;
	    memUsageTimestamp.dump();
	    blockUsageValue.dump();
	    memUsageValue.dump();
	}
	if (ALLOC_STATS && allocStats) {
	    Native.print_string("triPizlo: size overflow: ");
	    Native.print_int(sizeOverflow);
	    Native.print_string("\n");
	    Native.print_string("triPizlo: size histogram:\n");
	    for (int i=0;i<sizeHisto.length;++i) {
		Native.print_int(i);
		Native.print_string(" ");
		Native.print_int(sizeHisto[i]);
		Native.print_string("\n");
	    }
	}
	if (VERIFY_HEAP_INTEGRITY) {
	    if (!collecting) {
		verifyHeapIntegrity();
		Native.print_string("triPizlo: VERIFY_HEAP_INTEGRITY: doing a collection just to veriy heap integrity...\n");
		disableConcurrency=true;
		reallyCollect();
		Native.print_string("triPizlo: VERIFY_HEAP_INTEGRITY: collected.\n");
		verifyHeapIntegrity();
	    } else {
		Native.print_string("triPizlo: VERIFY_HEAP_INTEGRITY: not verifying heap integrity because we are in the middle of a collection.\n");
	    }
	}
	if (true) {
	    Native.print_string("triPizlo: total time spent in GC is ");
	    Native.print_long(totalGCTime);
	    Native.print_string(" ns\n");
	}
    }
    
    public boolean needsGCPauseTimer() { return true; }
    public boolean needsGCThread() { return true; }
    
    public void setShouldPause(boolean shouldPause) {
	if (DEBUG_PAUSE && debugPause) {
	    Native.print_string("triPizlo: PAUSE: setShouldPause called, shouldPause = ");
	    Native.print_string(shouldPause?"true":"false");
	    Native.print_string("\n");
	}
	synchronized (pollMonitor) {
	    this.shouldPause=shouldPause;
	    if (!shouldPause) {
		pollMonitor.notifyAll();
	    }
	}
    }
    
    static void printStats() {
	if (DEBUG_GC && debugGC) {
	    Native.print_string("triPizlo: GC: marked ");
	    Native.print_int(marked);
	    Native.print_string(" objects and waited ");
	    Native.print_int(waited);
	    Native.print_string(" times.\n");
	}
    }
    
    public boolean canCollect() { return threadRunning; }
    
    static Runnable poller=new Runnable(){
	    public void run() {
		pollSweeping();
	    }
	};

    static void zero(VM_Address addr,int nb) {
	Mem.the().zero(addr,nb,poller);
    }

    static void zeroBlock(int i) {
	zero(memory.add(i*blockSize),blockSize);
    }

    static void freeBlock(int i) {
	nUsedBlocks--;
	blockPrev[i]=-1;
	blockNext[i]=blockHead;
	blockPrev[blockHead]=i;
	blockHead=i;
    }
    
    static void walkStacks(int mode) {
	LocalReferenceIterator lri=LocalReferenceIterator.the();
	if (INCREMENTAL_STACK_SCAN) {
	    // QUESTION: what if a new thread is created during the
	    // stack scan?
	    // ANSWER: this is no different than a stack growing after
	    // it is scanned.
	    // FIXME: this code allows the weak tricolor invariant to
	    // be violated.  the stacks of all threads are grey in here
	    // until they are scanned.  imagine that a grey thread
	    // writes a pointer to some white object into a black object
	    // (possible, because objects are allocated black).  then
	    // we overwrite the reference on the stack to that white
	    // object.  if there are no other references to the white
	    // object, the weak tricolor invariant is violated.
            if (DEBUG_INTERRUPTIBLE_STACK_SCANNING) {
    	      Native.print_string("triPizlo: walkStacks: INCREMENTAL_STACK_SCAN\n");
            }
	    
	    for (Iterator i=Context.iterator();
		 i.hasNext();) {
		 
		pollStackScanning();
		final Context c=(Context)i.next();
		Iterator uniterator=new Iterator() {
			boolean has=true;
			public boolean hasNext() {
			    return has;
			}
			public Object next() {
			    has=false;
			    return c;
			}
			public void remove() {
			    throw new OVMError.Unimplemented("you've got to be kidding me");
			}
		    };
		lri.walkTheseContexts(uniterator,mode);
		while (lri.hasNext()) {
		    mark(lri.next().getAddress(),mode);

		}
	    }
	} else {
	    lri.walkCurrentContexts(mode);
	    while (lri.hasNext()) {
		mark(lri.next().getAddress(),mode);
	    }
	}
    }
    
    void reallyCollect() {
	outer.tic();
		
	collecting=true;
	allocColor=GREYBLACK;

	marked=0;
	waited=0;
	
	// in the predictable write barrier mode, it is necessary to reset the worklist before commencing GC
	worklistEmpty=true;
	worklistPuti=0;
	worklistGeti=0;

	marking=true; // if we are using the "unpredictable" write barrier, this enables it.

	if (DEBUG_GC && debugGC) {
	    Native.print_string("triPizlo: GC: collector has awakened.  starting with ");
	    Native.print_int(nUsedBlocks);
	    Native.print_string(" used blocks.\n");
	    
	    Native.print_string("triPizlo: GC: allocColor is now ");
	    Native.print_int(allocColor);
	    Native.print_string("\n");

	    if (!PREDICTABLE_BARRIER) {
		Native.print_string("triPizlo: GC: activating write barrier.\n");
	    }
	    
	    Native.print_string("triPizlo: GC: scanning stacks...\n");
	}
		
	// 1) scan stacks
	// warning: this code allocates memory.
	
	stackScanning = true; // if we are using the incremental stack scanning, and thus Dijsktra's barrier to
	                      // protect values from escaping from stacks before being scanned, this enables 
	                      // the barrier (in unpredictable mode)
	stackScan.tic();
	walkStacks(LocalReferenceIterator.CONSERVATIVE);
	walkStacks(LocalReferenceIterator.PRECISE);
	stackScan.toc("scanned stacks");
	stackScanning = false;
		
	gcStopped=false;

	printStats();

	if (DEBUG_GC && debugGC) {
	    Native.print_string("triPizlo: GC: stacks scanned.\n");
	}
		
	pollMarking();
		
	if (DEBUG_GC && debugGC) {
	    Native.print_string("triPizlo: GC: walking image...\n");
	}
		
	// 2) scan image
	// avoid walking the image header
	inner.tic();
	
//	if (useImageBarrier) {


          // first non-image-header byte of image 
          // (first byte of first object header in the image
        
        /* if scanning the first page, avoid walking image header */
        
        int idx;
        
        if (Bits.getBit(dirty, 0)) {
          idx = walkImageFromAddress( Native.getImageBaseAddress() );
        } else {
          idx = 1;
        }
        pollMarking();
        
        /* avoid scanning the last page, as image may end within it */ 
        
        // FIXME: the handling of image ending on page unaligned addres was
        // not here before the SW barrier ; is it really needed ?
        
        for(idx = Bits.findSet(dirty, nImgBitWords, idx); 
            (idx >= 0) && (idx < lastImagePageIdx);
            idx = Bits.findSet(dirty, nImgBitWords, idx)) {
            
            pollMarking();
            idx = walkImageFromPage( idx );
        }

        /* scan the last page, potentially a partial one */         
        
        if ( idx == lastImagePageIdx ) {
          mc.walk( pageIndexToImageAddress( lastImagePageIdx ), 
            Native.getImageEndAddress() );
        }
        
/*
	    int idx = 0;
	    if (Bits.getBit(dirty, idx)) {
		int sz = Bits.getSize(continued,idx);
		mc.walk(Native.getImageBaseAddress(),
			VM_Address.fromInt(getImageBaseAddress())
			.add(sz*imgPageSize));
		idx += sz;
	    }
	
	    for (idx = Bits.findSet(dirty, nImgBitWords, idx);
		 idx>=0;
		 idx = Bits.findSet(dirty, nImgBitWords, idx)) {
		pollMarking();
		if (Bits.getBit(continued,idx)) {
		    idx = Bits.rfindClr(continued,idx);
		}
		int sz = Bits.getSize(continued,idx);
		mc.walk(VM_Address.fromInt(getImageBaseAddress())
			.add(idx*imgPageSize),
			VM_Address.fromInt(getImageBaseAddress())
			.add((idx+sz)*imgPageSize));
		idx += sz;
	    }
	
	    // avoid walking the image trailer's magic word.
	    // FIXME: does this ever do anything at all?
	    if (idx > 0) {
		mc.walk(VM_Address.fromInt(getImageBaseAddress())
			.add(idx*imgPageSize),
			Native.getImageEndAddress());
	    }
*/
	    
//	} else {

/* 
  this would walk the whole image

	    mc.walk(Native.getImageBaseAddress(),
		    Native.getImageEndAddress());
	}
*/	
	inner.toc("scanned image");
	
	printStats();
	
	if (DEBUG_GC && debugGC) {
	    Native.print_string("triPizlo: GC: walking heap...\n");
	}
	
	// 3) walk the heap
	inner.tic();
	int walked=0;
	while (!worklistEmpty) {
	    pollMarking();
	    walkObject(worklistPop());
	    walked++;
	}
	inner.toc("walked heap");
	
	if (walked!=marked) {
	    throw Executive.panic("Did not walk all of the objects I marked.");
	}
	    
	printStats();

	marking=false;
		
	if (DEBUG_GC && debugGC) {
	    if (PREDICTABLE_BARRIER) {
		Native.print_string("triPizlo: GC: marking phase complete.\n");
	    } else {
		Native.print_string("triPizlo: GC: marking phase complete.  write barrier deactivated.\n");
	    }
	    Native.print_string("triPizlo: GC: sweeping...\n");
	}

	// 4) do sweeping
	inner.tic();
	int blocksFreed=0;
	int objsFreed=0;
	for (int i=Bits.findSet(usedBits,nBitWords,0);
	     i>=0;
	     i=Bits.findSet(usedBits,nBitWords,i)) {
	    if (DEBUG_SWEEP && debugSweep) {
		Native.print_string("triPizlo: SWEEP: sweeping block ");
		Native.print_int(i);
		Native.print_string("\n");
	    }
	    pollSweeping();
	    if (Bits.getBit(largeBits,i)) {
		if (sizes[i]<0) {
		    if (DEBUG_SWEEP && debugSweep) {
			Native.print_string("triPizlo: SWEEP: stumbled upon the continuation of a large object, stepping forward.\n");
		    }
		    i++;
		} else {
		    if (DEBUG_SWEEP && debugSweep) {
			Native.print_string("triPizlo: SWEEP: got ourselves a large object.\n");
		    }
		    VM_Address addr=memory.add(i*blockSize);
		    int size=sizes[i];
		    int bits=gcBits.getGCBits(addr);
		    if (bits==WHITE) {
			if (PROFILE_MEM_USAGE) {
			    memUsage-=objectSize(addr);
			    observeMemUsageChanged();
			}
			// free it
			if (DEBUG_FREE && debugFree) {
			    Native.print_string("triPizlo: FREE: freeing large object at ");
			    Native.print_ptr(memory.add(i*blockSize));
			    Native.print_string("\n");
			}
			for (int j=size+i;j-->i;) {
			    pollSweeping();
			    zeroBlock(j);
			    Bits.clrBit(usedBits,j);
			    Bits.clrBit(largeBits,j);
			    blocksFreed++;
			    freeBlock(j);
			    observeMemUsageChanged();
			}
		    }
		    i+=size;
		}
	    } else {
		if (DEBUG_SWEEP && debugSweep) {
		    Native.print_string("triPizlo: SWEEP: got ourselves a small object block.\n");
		}
		// small object sweep
		int allocSize=sizes[i];
		int nObjs=blockSize/allocSize;
		VM_Address block=memory.add(i*blockSize);
		    
		SizeClass sc=sizeClassBySize[allocSize/alignment];

		VM_Address cur=block;
		for (int j=nObjs;j-->0;cur=cur.add(allocSize)) {
		    pollSweeping();
		    if (gcBits.getGCBits(cur)==WHITE) {
			uses[i]--;
		    }
		}
		
		if (sc.block==block) {
		    if (DEBUG_SWEEP && debugSweep) {
			Native.print_string("triPizlo: SWEEP: block is in use for pointer-bump allocation, turning it into a normal block.\n");
		    }
		    if (sc.blockCur==null) {
			throw Executive.panic("blockCur is null when block is non-null.");
		    }
		    // turn off pointer-bump for this block
		    VM_Address blockCur=sc.blockCur;
		    sc.block=null;
		    sc.blockCur=null;
		    // and now flip the bits, in an interruptible way
		    for (cur=blockCur;
			 block.add(blockSize).diff(cur).asInt()
			     >= allocSize;
			 cur=cur.add(allocSize)) {
			pollSweeping();
			gcBits.setGCBits(cur,WHITE);
		    }
		}
		
		if (uses[i]<0) {
		    throw Executive.panic("uses went negative");
		}

		if (uses[i]==0) {
		    // block seems to be empty, try to remove objects from
		    // freelist.  the tricky thing is that the mutator could
		    // interrupt us and allocate one of these objects, in
		    // which case we bail.
		    int succMemUsageDiff=0;
		    int failMemUsageDiff=0;
		    if (DEBUG_FREE && debugFree) {
			Native.print_string("triPizlo: FREE: freeing block ");
			Native.print_ptr(block);
			Native.print_string("\n");
		    }
		    // free block
		    cur=block;
		    for (int j=nObjs;j-->0;cur=cur.add(allocSize)) {
			pollSweeping();
			if (uses[i]>0) {
			    if (DEBUG_WEIRD && debugWeird) {
				Native.print_string("triPizlo: WEIRD: aborting block free because of allocation\n");
			    }
			    break;
			}
			int bits=gcBits.getGCBits(cur);
			if (bits==FREE) {
			    // code to remove from freelist
			    if (PROFILE_MEM_USAGE) {
				failMemUsageDiff+=objectSize(cur);
			    }
			    if (freeList.getPrev(cur)==null) {
				sc.objHead=freeList.getNext(cur);
			    } else {
				freeList.setNext(freeList.getPrev(cur),
						 freeList.getNext(cur));
			    }
			    if (freeList.getNext(cur)!=null) {
				freeList.setPrev(freeList.getNext(cur),
						 freeList.getPrev(cur));
			    }
			    gcBits.setGCBits(cur,WHITE);
			} else if (PROFILE_MEM_USAGE && bits==WHITE) {
			    succMemUsageDiff-=objectSize(cur);
			}
		    }
		    if (uses[i]==0) {
			zeroBlock(i);
			Bits.clrBit(usedBits,i);
			freeBlock(i);
			blocksFreed++;
			if (PROFILE_MEM_USAGE) {
			    memUsage+=succMemUsageDiff;
			}
			observeMemUsageChanged();
		    } else if (PROFILE_MEM_USAGE) {
			memUsage+=failMemUsageDiff;
		    }
		}
		
		if (uses[i]>0) {
		    if (DEBUG_SWEEP && debugSweep) {
			Native.print_string("triPizlo: SWEEP: adding free objects in block to freelist.\n");
		    }
		    // add free objects to freelist
		    cur=block;
		    for (int j=nObjs;j-->0;cur=cur.add(allocSize)) {
			pollSweeping();
			int bits=gcBits.getGCBits(cur);
			if (DEBUG_SWEEP && debugSweep) {
			    Native.print_string("triPizlo: SWEEP: object at ");
			    Native.print_ptr(cur);
			    Native.print_string(" has bits ");
			    Native.print_int(bits);
			    Native.print_string("\n");
			}
			if (bits==WHITE) {
			    if (DEBUG_FREE && debugFree) {
				Native.print_string("triPizlo: FREE: freeing object ");
				Native.print_ptr(cur);
				Native.print_string("\n");
			    }
			    if (PROFILE_MEM_USAGE) {
				memUsage-=objectSize(cur);
			    }
			    zero(cur,allocSize);
			    freeList.setPrev(cur,null);
			    freeList.setNext(cur,sc.objHead);
			    if (sc.objHead!=null) {
				freeList.setPrev(sc.objHead,cur);
			    }
			    sc.objHead=cur;
			    gcBits.setGCBits(cur,FREE);
			    objsFreed++;
			    if (PROFILE_MEM_USAGE) {
				observeMemUsageChanged();
			    }
			}
		    }
		}

		i++;
	    }
	}
	inner.toc("swept");
		
	collecting=false;
	
	if (DEBUG_GC && debugGC) {
	    Native.print_string("triPizlo: GC: waited ");
	    Native.print_int(waited);
	    Native.print_string(" times.\n");
		
	    Native.print_string("triPizlo: GC: freed ");
	    Native.print_int(blocksFreed);
	    Native.print_string(" blocks and ");
	    Native.print_int(objsFreed);
	    Native.print_string(" objects.  there are now ");
	    Native.print_int(nUsedBlocks);
	    Native.print_string(" blocks used.\n");
	}

	// there are no objects in the heap that are white at this moment.
	// we swap the meaning of GREYBLACK and WHITE so that all GREYBLACK
	// objects become WHITE.
	swapColors();
	if (DEBUG_GC && debugGC) {
	    Native.print_string("triPizlo: GC: GREYBLACK is now ");
	    Native.print_int(GREYBLACK);
	    Native.print_string(" and WHITE is now ");
	    Native.print_int(WHITE);
	    Native.print_string("\n");
	}
	
	// this does nothing, but I leave it in to illustrate that the
	// allocColor is now WHITE.
	allocColor=WHITE;
	if (DEBUG_GC && debugGC) {
	    Native.print_string("triPizlo: GC: allocColor is now ");
	    Native.print_int(allocColor);
	    Native.print_string("\n");
	}
	
	if (VERIFY_HEAP_INTEGRITY) {
	    for (int i=Bits.findSet(usedBits,nBitWords,0);
		 i>=0;
		 i=Bits.findSet(usedBits,nBitWords,i)) {
		Native.print_string("VERIFY_HEAP_INTEGRITY: examining block ");
		Native.print_int(i);
		Native.print_string("\n");
		VM_Address block=memory.add(i*blockSize);
		int size=sizes[i];
		if (Bits.getBit(largeBits,i)) {
		    if (gcBits.getGCBits(block)==GREYBLACK) {
			throw Executive.panic("VERIFY_HEAP_INTEGRITY: ERROR: there is a large GREYBLACK object in the heap after the sweep.\n");
		    }
		    i+=size;
		} else {
		    int nObjs=blockSize/size;
		    VM_Address cur=block;
		    for (int j=nObjs;j-->0;cur=cur.add(size)) {
			Native.print_string("VERIFY_HEAP_INTEGRITY: looking at object ");
			Native.print_ptr(cur);
			Native.print_string(", which has bits ");
			Native.print_int(gcBits.getGCBits(cur));
			Native.print_string("\n");
			if (gcBits.getGCBits(cur)==GREYBLACK) {
			    throw Executive.panic("VERIFY_HEAP_INTEGRITY: ERROR: there is a small GREYBLACK object in the heap after the sweep.\n");
			}
		    }
		    i++;
		}
	    }
	}
	    
	if (DEBUG_GC && debugGC) {
	    Native.print_string("triPizlo: GC: sweeping done.  collector going to sleep.\n");
	}
		
	outer.toc("collected");
	totalGCTime+=outer.lat-outer.lost;
	
	gcStopped=true;
    }
    
    static void swapColors() {
	int tmp=GREYBLACK;
	GREYBLACK=WHITE;
	WHITE=tmp;
    }
    
    private static void verifyFreeObject(VM_Address cur) {
	if (gcBits.getGCBits(cur)!=FREE) {
	    Native.print_string("ERROR: object in freelist is not free.\n");
	}
	if (!Bits.getBit(usedBits,blockIdx(cur))) {
	    Native.print_string("ERROR: block holding object in freelist is not in use.\n");
	}
	if (Bits.getBit(largeBits,blockIdx(cur))) {
	    Native.print_string("ERROR: block holding object in freelist is a large object block.\n");
	}
    }
    
    private static void verifyFreelistIntegrity(SizeClass sc) {
	VM_Address cur=sc.objHead;
	if (cur!=null) {
	    if (freeList.getPrev(cur)!=null) {
		Native.print_string("ERROR: first object in freelist has non-null prev pointer.\n");
	    }
	    for (;;) {
		verifyFreeObject(cur);
		VM_Address next=freeList.getNext(cur);
		if (next==null) {
		    break;
		}
		if (cur!=freeList.getPrev(next)) {
		    Native.print_string("ERROR: object in freelist has incorrect prev pointer.\n");
		}
		cur=next;
	    }
	}
    }
    
    private static void verifyPointerBumpIntegrity(SizeClass sc) {
	if (sc.block!=null) {
	    Native.print_string("Doing pointer bump allocation in block ");
	    Native.print_ptr(sc.block);
	    Native.print_string(" (");
	    Native.print_int(blockIdx(sc.block));
	    Native.print_string(") for size class ");
	    Native.print_int(sc.size);
	    Native.print_string("\n");
	    for (VM_Address cur=sc.block;
		 cur.uLT(sc.blockCur);
		 cur=cur.add(sc.size)) {
		if (gcBits.getGCBits(cur)==FREE) {
		    Native.print_string("ERROR: object allocated using pointer-bump GC is free.\n");
		}
	    }
	    for (VM_Address cur=sc.blockCur;
		 cur.uLE(sc.block.add(blockSize).sub(sc.size));
		 cur=cur.add(sc.size)) {
		if (gcBits.getGCBits(cur)!=0) {
		    Native.print_string("ERROR: free object in pointer-bump allocated block has non-zero GC bits.\n");
		}
	    }
	}
    }
    
    private static void verifyFreelistIntegrity() {
	for (int sci=0;sci<sizeClasses.length;++sci) {
	    verifyPointerBumpIntegrity(sizeClasses[sci]);
	    verifyFreelistIntegrity(sizeClasses[sci]);
	}
    }
    
    private static void verifyBlockIntegrity() {
	for (int cur=0;cur<nBlocks;) {
	    if (Bits.getBit(usedBits,cur)) {
		if (Bits.getBit(largeBits,cur)) {
		    if (gcBits.getGCBits(memory.add(cur*blockSize))==FREE) {
			Native.print_string("ERROR: large object is free.\n");
		    }
		    int nBlocks=sizes[cur];
		    for (int i=cur+1;i<cur+nBlocks;++i) {
			if (!Bits.getBit(usedBits,i)) {
			    Native.print_string("ERROR: large object continuation is not marked used.\n");
			}
			if (!Bits.getBit(largeBits,i)) {
			    Native.print_string("ERROR: large object continuation is not marked large.\n");
			}
			if (sizes[i]!=-1) {
			    Native.print_string("ERROR: large object continuation is not marked as a continuation.\n");
			}
		    }
		    cur+=nBlocks;
		} else {
		    int allocSize=sizes[cur];
		    int nObjs=blockSize/allocSize;
		    VM_Address ptr=memory.add(blockSize*cur);
		    SizeClass sc=sizeClassBySize[allocSize/alignment];
		    for (int i=nObjs;i-->0;ptr=ptr.add(allocSize)) {
			if (gcBits.getGCBits(ptr)==FREE) {
			    if (freeList.getPrev(ptr)==null) {
				if (sc.objHead!=ptr) {
				    Native.print_string("ERROR: prev==null but we are not the head.\n");
				}
			    } else if (freeList.getNext(freeList.getPrev(ptr))!=ptr) {
				Native.print_string("ERROR: prev->next != this\n");
			    }
			    if (freeList.getNext(ptr)!=null &&
				freeList.getPrev(freeList.getNext(ptr))!=ptr) {
				Native.print_string("ERROR: next->prev != this\n");
			    }
			}
		    }
		    cur++;
		}
	    } else {
		if (Bits.getBit(largeBits,cur)) {
		    Native.print_string("ERROR: large object block is not marked used.\n");
		}
		cur++;
	    }
	}
    }
    
    private static void verifyBlockFreelistIntegrity() {
	int prev=-1;
	for (int cur=blockHead;cur>=0;cur=blockNext[cur]) {
	    if (prev!=blockPrev[cur]) {
		Native.print_string("ERROR: invalid prev pointer in block freelist.\n");
	    }
	    if (Bits.getBit(usedBits,cur)) {
		Native.print_string("ERROR: block in block freelist is in use.\n");
	    }
	    if (Bits.getBit(largeBits,cur)) {
		Native.print_string("ERROR: block in block freelist is a large object block.\n");
	    }
	    prev=cur;
	}
    }
    
    static void verifyHeapIntegrity() throws PragmaAtomic {
	// things this doesn't check for:
	// - blockCur (pointer bump allocation)
	// - errors that happen just before the crash

	Native.print_string("triPizlo: VERIFY_HEAP_INTEGRITY: running...\n");
	verifyFreelistIntegrity();
	verifyBlockIntegrity();
	verifyBlockFreelistIntegrity();
	Native.print_string("triPizlo: VERIFY_HEAP_INTEGRITY: done running.\n");
    }
    
    public void doGC() {
	for (;;) {
	    pollLat.tic();
	    synchronized (pollMonitor) {
		while (nUsedBlocks+gcThreshold < nBlocks) {
		    try {
			pollMonitor.wait();
		    } catch (InterruptedException e) {
			throw Executive.panic("I just got an InterruptedException in doGC()");
		    }
		}
	    }
	    pollLat.toc();
	    
	    reallyCollect();
	    
	    if (VERIFY_HEAP_INTEGRITY) {
		verifyHeapIntegrity();
	    }
	}
    }

    public void runGCThread() {
	try {
	    threadRunning=true;
	    if (DEBUG_GC && debugGC) {
		Native.print_string("triPizlo: GC: GC thread running\n");
	    }
	    LocalReferenceIterator.the().prepareForGC();
	} catch (Throwable e) {
	    Executive.panicOnException(e,"In gcThreadMain");
	}
    }

    static void markUnchecked(VM_Address addr)
	throws PragmaNoPollcheck {
	if (DEBUG_MARK && debugMark) {
	    Native.print_string("triPizlo: MARK: marking ");
	    Native.print_ptr(addr);
	    Native.print_string("\n");
	}
	gcBits.setGCBits(addr,GREYBLACK);
	if (!worklistEmpty && worklistPuti==worklistGeti) {
	    throw Executive.panic("Ran out of worklist space in the GC");
	}
	worklist.add(worklistPuti*wordSize).setAddress(addr);
	worklistEmpty=false;
	worklistPuti++;
	worklistPuti&=worklistBits;
	marked++;
    }
    
    static void markNonNull(VM_Address addr) throws PragmaNoPollcheck {
	if (DEBUG_MARK_VERBOSE && debugMarkVerbose) {
	    Native.print_string("triPizlo: MARK_VERBOSE: attempting to mark ");
	    Native.print_ptr(addr);
	    Native.print_string("\n");
	}
	if (VERIFY_HEAP_INTEGRITY && inHeap(addr)) {
	    verifyPointer(addr);
	}
	if (gcBits.getGCBits(addr)==WHITE) {
	    markUnchecked(addr);
	}
    }
    
    static void mark(VM_Address addr) throws PragmaNoPollcheck {
	if (addr!=null) {
	    markNonNull(addr);
	}
    }
    
    static void markAmbiguous(VM_Address addr) throws PragmaNoPollcheck {
	int offset=offset(addr);
	int block=blockIdxOff(offset);
	if (block>=0 && block<nBlocks
	    && Bits.getBit(usedBits,block)) {
	    int size=sizes[block];
	    if (Bits.getBit(largeBits,block)) {
		while (size<0) {
		    block--;
		    size=sizes[block];
		}
		markNonNull(memory.add(block*blockSize));
	    } else {
		// this works for bump-pointer blocks because all objects
		// above the pointer are PINNED (because PINNED is 0)
		
		VM_Address objAddr = memory.add(((offset-block*blockSize)/size)*size+block*blockSize);
		
		
                // calling markNonNull on a FREE object is not in general a problem, because a FREE object is not WHITE,
                // and thus will not be really marked,
                // but, with VERIFY_HEAP_INTEGRITY, an error would be reported anyway, which is confusing
                
		if ( (!VERIFY_HEAP_INTEGRITY) || (gcBits.getGCBits(objAddr)!=FREE) ) {
    		  markNonNull(objAddr);
                }
	    }
	}
    }
    
    static void mark(VM_Address addr,
		     int mode) throws PragmaNoPollcheck {
	if (mode==LocalReferenceIterator.PRECISE) {
	    mark(addr); 
	} else {
	    markAmbiguous(addr);
	}
    }
    
    static VM_Address worklistPop()
	throws PragmaNoPollcheck {
	VM_Address result=worklist.add(worklistGeti*wordSize).getAddress();
	worklistGeti++;
	worklistGeti&=worklistBits;
	worklistEmpty=(worklistGeti==worklistPuti);
	return result;
    }
    
    static void storeWithoutBarrier( Oop referee,int refOffset,Oop newReferent ) {
    
      VM_Address ptrPtr=VM_Address.fromObject(referee).add(refOffset);
      ptrPtr.setAddress(VM_Address.fromObject(newReferent));
    }
    
    
    static void markFromBarrier( VM_Address addr ) throws PragmaNoPollcheck, PragmaInline {
    
        // when not marking, every non-null object is white.
	if (addr!=null && gcBits.getGCBits(addr)==WHITE) {

	  // when marking, set to black; when not marking, set to
	  // white (so no change)
	  gcBits.setGCBits(addr ,allocColor);
	  
          // when not marking, this happily corrupts the worklist,
          // but we don't care - it is simply reset at the beginning
          // of the next GC.
		
          worklist.add(worklistPuti*wordSize).setAddress(addr);
          worklistEmpty=false;
          worklistPuti++;
          worklistPuti&=worklistBits;
          marked++;
        }
    }
    
    static void storeBarrier(Oop referee,int refOffset,Oop newReferent)
	throws PragmaNoPollcheck, PragmaInline {
	// what happens when we are scanning an array?
	// answer: it just works.  the thing is, we don't ever have to
	// know the difference between grey and black.


        /* SW image barrier */	
	int ipIdx = imageAddressToPageIndex(VM_Address.fromObject(referee));

	if ( (ipIdx >=0) && (ipIdx <= lastImagePageIdx ) ) {
	  Bits.setBit(dirty,ipIdx);
	}
		
	VM_Address ptrPtr=VM_Address.fromObject(referee).add(refOffset);
	
	if (PREDICTABLE_BARRIER) {
	    // this barrier is designed to have the same performance
	    // whether it is active or not.

	    VM_Address oldObj = ptrPtr.getAddress();
	    if (DEBUG_BARRIER && debugBarrier) {
		Native.print_string("triPizlo: BARRIER: dealing with old referrent ");
		Native.print_ptr(oldObj);
		Native.print_string("\n");
	    }

	    markFromBarrier(oldObj);
	    
	    if (INCREMENTAL_STACK_SCAN && (newReferent!=null) ) {

              // FIXME: is .getAddress() needed ?
              VM_Address newObj = VM_Address.fromObject(newReferent).getAddress();

              if (DEBUG_BARRIER && debugBarrier) {
                Native.print_string("triPizlo: BARRIER: dealing with new referrent ");
		Native.print_ptr(newObj);
		Native.print_string("\n");
              }

              markFromBarrier(newObj);
	    }
	    
	    if (DEBUG_BARRIER && debugBarrier) {
		Native.print_string("triPizlo: BARRIER: done.\n");
	    }
	    
	} else {
	    if (marking) {
  	    
              VM_Address oldObj=ptrPtr.getAddress();  	    

  	      if (DEBUG_BARRIER && debugBarrier) {
		Native.print_string("triPizlo: BARRIER: dealing with old referrent (non-predictable barrier)");
		Native.print_ptr(oldObj);
		Native.print_string("\n");
              }

              mark(oldObj);

              if (INCREMENTAL_STACK_SCAN && stackScanning && (newReferent!=null)) {

                // FIXME: is .getAddress() needed ?
                VM_Address newObj = VM_Address.fromObject(newReferent).getAddress();

    	        if (DEBUG_BARRIER && debugBarrier) {
		  Native.print_string("triPizlo: BARRIER: dealing with new referrent (non-predictable barrier)");
		  Native.print_ptr(newObj);
		  Native.print_string("\n");
                }              

	        mark(newObj);
              }
	    }
	}
	ptrPtr.setAddress(VM_Address.fromObject(newReferent));
    }
    
    public void putFieldBarrier(CoreServicesAccess csa,
				Oop src,int offset, Oop tgt)
	throws PragmaNoPollcheck, PragmaInline {
	storeBarrier(src,offset,tgt);
    }

    public void aastoreBarrier(CoreServicesAccess csa,
			       Oop src,int offset, Oop tgt)
	throws PragmaNoPollcheck, PragmaInline {
	storeBarrier(src,offset,tgt);
    }
    
    public boolean forceBarriers() {
	return true;
    }

    public boolean needsWriteBarrier() {
//	return !noBarrier;
        return true;
    }
    
    public boolean needsReadBarrier() {
	return VERIFY_HEAP_INTEGRITY;
    }
    
    public void readBarrier(CoreServicesAccess csa,
			    Oop src) throws PragmaNoPollcheck {
	if (inHeap(src)) {
	    verifyPointer(src);
	}
    }
    
    public VM_Area getHeapArea() {
	return heap;
    }
    
    public int allocateInImage(int firstFree,
			       int size,
			       Blueprint bp,
			       int arrayLength) throws BCdead {
	if (intArray==null) {
	    initLate();
	}
	if (!lastBig
	    && ((firstFree + size)/imgPageSize) == (firstFree/imgPageSize))
	    return firstFree;
	else if (size <= imgPageSize) {
	    lastBig = false;
	    return ((firstFree + imgPageSize - 1) & ~(imgPageSize - 1));
	} else {
	    lastBig = true;
	    int ret = (firstFree + imgPageSize - 1) & ~(imgPageSize - 1);
/*	    params.println("\t{ 0x" + Integer.toHexString(ret) +
			     ", 0x" + Integer.toHexString(ret+size) + "}, \\");
*/			     
	    return ret;
	}
    }
    
    public void garbageCollect() {}

    public boolean shouldPinCrazily() { return false; }

    // FIXME: when you pin an object, its fields may not get scanned.
    public void pin(Oop oop) throws PragmaAtomic {
	if (DEBUG_PIN && debugPin) {
	    Native.print_string("triPizlo: PIN: pinning ");
	    Native.print_ptr(VM_Address.fromObject(oop));
	    Native.print_string("\n");
	}
	if (OVMBase.isBuildTime() || !inHeap(oop)) {
	    // For side effect
	    VM_Address.fromObject(oop).asInt();
	    if (DEBUG_PIN && debugPin) {
		Native.print_string("triPizlo: PIN: skipping, not in heap.\n");
	    }
	    return;
	}
	GCBits goop=(GCBits)oop.asAnyOop();
	if (goop.getGCBits()!=WHITE &&
	    goop.getGCBits()!=GREYBLACK) {
	    Native.print_string("triPizlo: PIN: object has GC bits ");
	    Native.print_int(goop.getGCBits());
	    Native.print_string("\n");
	    throw Executive.panic("Attempt to pin an object that is neither "+
				  "white, grey, or black.");
	}
	goop.setGCBits(PINNED);
    }
    public void unpin(Oop oop) throws PragmaAtomic {
	if (DEBUG_PIN && debugPin) {
	    Native.print_string("triPizlo: PIN: unpinning ");
	    Native.print_ptr(VM_Address.fromObject(oop));
	    Native.print_string("\n");
	}
	if (OVMBase.isBuildTime() || !inHeap(oop)) {
	    if (DEBUG_PIN && debugPin) {
		Native.print_string("triPizlo: PIN: skipping, not in heap.\n");
	    }
	    return;
	}
	GCBits goop=(GCBits)oop.asAnyOop();
	if (goop.getGCBits()!=PINNED) {
	    throw Executive.panic("Attempt to unpin an object that is not "+
				  "pinned.");
	}
	stampGCBits(goop);
    }
    
    public boolean isPrecise() { return true; }
    
}


